
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;

import com.ibm.ccd.ui.util.UIUtils;


/**
 *
 * Represents the base attribute class for the WPC data types
 * which contain the metadata info like type, constraints and values
 * The attribute values are saved in the valueList irrespective of the data types
 */
public abstract class Attribute implements Cloneable, Serializable {
    // attribute name
    private String name;

    // flag to check whethere this is a sub-attribute 
    private boolean subAttribute;

    // minimum occurrences of this attribute    
    protected int minimumOccurrence;

    //	maximum occurrences of this attribute
    protected int maximumOccurrence;

    //current occurrence of this attribute
    protected int currOccurrence;

    // minimum length of the attribtue value
    protected int minimumLength;

    //	minimum length of the attribtue value
    protected int maximumLength;

    // represents whether this attribute is editable or not
    protected boolean editable;

    // represents the path of the attribute 
    private String path;

    // atribute values list	
    protected ArrayList valueList;

    // an indicator that this Attribute is the primary key. This can be used by the User Interface as a unique identifier for data elements
    protected boolean isPrimaryKey = false;

    // an indicator that this Attribute if from the Category Specific Attribute list
    protected boolean isCategorySpecificAttribute = false;

    // the attribute label is fetched from WPC, and contains a locale specific name
    protected String displayLabelName;

    // the attribute is marked as a localized one, if it contains locale specific content from WPC.
    protected boolean isLocalized = false;

    // this will contain locale specific values for an attribute.
    protected ArrayList localizedValues = new ArrayList();
	
	// parent attribute of this attribute  
	protected Attribute parentAttribute;	
	
	// determins the parentpath of the attribute
	protected String parentPath;
	
	protected String hashName;

    // locale setting for the attribute
    private String localeName;

	// determins attribute value is changed
	private boolean isValueChanged;
	
	// Hashmap for storing the error message for each attribute occurence
	protected HashMap errorMap;
	
    public Attribute(String name, boolean editable, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        this(name, editable, 0, 0, minimumOccurrence, maximumOccurrence,
            displayName);
    }

    public Attribute(String name, boolean editable, int minimumLength,
        int maximumLength, int minimumOccurrence, int maximumOccurrence,
        String displayName) {
        valueList = new ArrayList();
        this.name = name;
        this.editable = editable;
        this.minimumLength = minimumLength;
        this.maximumLength = maximumLength;
        this.minimumOccurrence = minimumOccurrence;
        this.maximumOccurrence = maximumOccurrence;
        this.currOccurrence = (this.minimumOccurrence == 0) ? 1
                                                            : this.minimumOccurrence;
        this.displayLabelName = displayName;
        errorMap = new HashMap();
    }

    public boolean isEditable() {
        return editable;
    }

    public int getMaximumLength() {
        return maximumLength;
    }

    public int getMaximumOccurrence() {
        return maximumOccurrence;
    }

    public int getMinimumLength() {
        return minimumLength;
    }

    public int getMinimumOccurrence() {
        return minimumOccurrence;
    }

    public String getName() {
        return name;
    }

    public void setEditable(boolean editable) {
        this.editable = editable;
    }

    public void setMaximumLength(int maximumLength) {
        this.maximumLength = maximumLength;
    }

    public void setMaximumOccrrence(int maximumOccurrence) {
        this.maximumOccurrence = maximumOccurrence;
    }

    public void setMinimumLength(int minimumLength) {
        this.minimumLength = minimumLength;
    }

    public void setMinimumOccrrence(int minimumOccurrence) {
        this.minimumOccurrence = minimumOccurrence;
    }

    public void setName(String name) {
        this.name = name;
    }

    /*
     * represents whether this attribute is configured for multi occurrences or not
     */
    public boolean isMultiValued() {
        return (maximumOccurrence > 1);
    }

    /*
    *  returns the current occurrence size ( or the size of values set)
    */
    public int getCurrentOccurrence() {
        return valueList.size();
    }

    /*
     * represents the type of attribute
     * concrete attributes has to impl. this method to represent their types
     */
    public abstract int getType();

    public ArrayList getValueList() {
        return valueList;
    }

    public void setValueList(ArrayList valueList) {
        this.valueList = valueList;
    }

    public boolean isSubAttribute() {
        return subAttribute;
    }

    public void setSubAttribute(boolean subAttribute) {
        this.subAttribute = subAttribute;
    }

    /*
     * Clears the valueList
     */
    public void removeValues() {
        if (valueList != null) {
            valueList.clear();
            errorMap.clear();
        }
    }

    /*
     * Clears the appropriate occurrence value
     */
    public void removeValue(int occurrence) {
    	
    	/*
    	 * Change History : for CQ WPC00024603
    	 * changed ">=" to ">" 
    	 */
        if ((valueList != null) && (valueList.size() > occurrence)) {
            valueList.remove(occurrence);
            String key = getParentPath()+"_"+getHashName() + "_" + occurrence;
            errorMap.remove(key);
            /*
             * In a multioccurence scenario for number attribute, if the nth occurence is being removed 
             * and if the n+1 th occurence is invalid then when the attribute is being rendered again, the renderElement
             * method will not find any error for the nth occurence but the new nth occurence value will be the
             * invalid n+1th occurence value. While rendering this invalid value it will try to create a double object
             * and fail. To prevent this the errorMap needs to be reset by moving all the n+i entries to n entries.
             */
            resetErrorMap(occurrence);
        }
    }
    
    private void resetErrorMap(int occurence){
    	int size = errorMap.size();
    	for(int i=occurence;i<size;i++){
    		String key = getParentPath()+"_"+getHashName() + "_" + i;
    		int newIndex = i + 1;
    		String nextKey = getParentPath()+"_"+getHashName() + "_" + newIndex;
    		errorMap.put(key,(String)errorMap.get(nextKey));
    	}
    }

    /**
     * @return primaryKey
     */
    public boolean isPrimaryKey() {
        return isPrimaryKey;
    }

    /**
     * when minOccurrence of an attribute is equals 0 return true
     * otherwise false
     * @return boolean
     */
    public boolean isOptional() {
        if (minimumOccurrence == 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param primaryKey
     */
    public void setPrimaryKey(boolean primaryKey) {
        isPrimaryKey = primaryKey;
    }

    /**
     * @return categorySpecificAttrib
     */
    public boolean isCategorySpecificAttribute() {
        return isCategorySpecificAttribute;
    }

    /**
     * @param categorySpecificAttrib
     */
    public void setCategorySpecificAttribute(boolean categorySpecificAttrib) {
        isCategorySpecificAttribute = categorySpecificAttrib;
    }

    /*
    * Prints the attribute details
    */
    public void printMe() {
        System.out.print("Attribute Name:" + name + " Type: " + getType() +
            " :" + AttributeTypes.getString(getType()));
        System.out.print(" Editable :" + editable + " minOcc: " +
            minimumOccurrence + " maxocc: " + maximumOccurrence);
        System.out.println(" Min.Length :" + minimumLength + " MaxLeng:" +
            maximumLength);

        if ((valueList == null) || (valueList.size() == 0)) {
            System.out.println("\t\tNo Value Set...");

            return;
        }

        System.out.print("\t\tValue(s):");

        for (int i = 0; i < valueList.size(); ++i) {
            System.out.print("\t" + valueList.get(i));
        }

        System.out.println();
    }

    /**
     * @return
     */
    public int getCurrOccurrence() {
        return (valueList.size() == 0) ? 1 : valueList.size();
    }

    /**
     * @param i
     */
    public void setCurrOccurrence(int i) {
        currOccurrence = i;
    }

    /**
     * @return
     */
    public String getPath() {
        return path;
    }

    /**
     * @param string
     */
    public void setPath(String string) {
        path = string;
    }

    /**
     * @return Returns the displayLabelName.
     */
    public String getDisplayLabelName() {
        return (UIUtils.isEmpty(displayLabelName)) ? name : displayLabelName;
    }

    /**
     * @param displayLabelName The displayLabelName to set.
     */
    public void setDisplayLabelName(String displayLabelName) {
        this.displayLabelName = displayLabelName;
    }

    public void setAsLocalized(boolean isLocalized) {
        this.isLocalized = isLocalized;
    }

    public boolean isLocalized() {
        return isLocalized;
    }

    /**
     * @return Returns the localizedValues.
     */
    public ArrayList getLocalizedValues() {
        return localizedValues;
    }

    /**
     * @param localizedValues The localizedValues to set.
     */
    public void setLocalizedValues(ArrayList localizedValues) {
        this.localizedValues = localizedValues;
    }

    /**
     * @return Returns the localeName.
     */
    public String getLocaleName() {
        return localeName;
    }

    /**
     * @param localeName The localeName to set.
     */
    public void setLocaleName(String localeName) {
        this.localeName = localeName;
    }
	public Attribute getParentGroup() {
		return parentAttribute;
	}
	
	public void setParentGroup(Attribute parentAttribute) {
		this.parentAttribute = parentAttribute;
	}

	protected String prepareAttributeParentPath() {
			
		if(getParentGroup() != null) {
			GroupAttribute groupAttrib = (GroupAttribute)getParentGroup();
			int groupIdx = groupAttrib.getGroupIndex();
			if(groupIdx == -1) 
					groupIdx = 0;
			return groupIdx+"_"+ groupAttrib.getHashName() + "_" + groupAttrib.prepareAttributeParentPath();
		}else {
			return "";
		}
	}
	public String getAttributeParentPath() {
		
		StringTokenizer tokenizer = new StringTokenizer(prepareAttributeParentPath(),"_");
		ArrayList list = new ArrayList();
		
		while(tokenizer.hasMoreTokens()) {
			list.add(tokenizer.nextToken());
		}
		
		String parentPath ="";
		for(int i=list.size()-1;i>=0;i--) {
			parentPath+="_"+(String)list.get(i);
		}
		
		return parentPath;
	}
	/**
	 * @return
	 */
	public String getParentPath() {
		if(parentPath != null)
			return parentPath;
		else 
			return parentPath=getAttributeParentPath();	
	}

	/**
	 * @param string
	 */
	public void setParentPath(String string) {
		parentPath = string;
	}
	/**
	 * @return
	 */
	public String getHashName() {
		if(name != null) {
			return String.valueOf(Math.abs(name.hashCode()));
		}
		else 
			return "";	
	}

	/**
	 * @param string
	 */
	public void setHashName(String string) {
		hashName = string;
	}
	/**
	 * @return
	 */
	public boolean isValueChanged() {
		return isValueChanged;
	}

	/**
	 * @param b
	 */
	public void setValueChanged(boolean b) {
		isValueChanged = b;
	}
	
	public boolean hasAnyParentGroupMultivalued() {
		if(parentAttribute != null) {
			GroupAttribute group = (GroupAttribute) parentAttribute;
			if(group.isMultiValued()) {
				return true;
			}else {
				group.hasAnyParentGroupMultivalued();
			}
		}
		return false;	 	
	}
	
	public void setParentGroupFlagValueChanged() {
		if(parentAttribute != null) {
			parentAttribute.setValueChanged(true);
			parentAttribute.setParentGroupFlagValueChanged();			
		}
	}
	
	public void setValidationError(String id,String error) {
		errorMap.put(id,error);
	}
	
	public String getValidationError(String id){
		return (String)errorMap.get(id);
	}
}
