
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.items;

import java.io.File;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

import com.ibm.ccd.ui.cache.CacheConstants;
import com.ibm.ccd.ui.cache.CacheFactory;
import com.ibm.ccd.ui.cache.CacheManager;
import com.ibm.ccd.ui.cache.CacheNotEnabledException;
import com.ibm.ccd.ui.cache.ICacheFactory;
import com.ibm.ccd.ui.dataobjects.FileUploadResource;
import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.BinaryAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.CurrencyAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.DateAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.FlagAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.GroupAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.IntegerAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.LookupTableAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PasswordAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PeriodAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.RelationshipAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.URLAttribute;
import com.ibm.ccd.ui.dataobjects.workflow.CollaborationArea;
import com.ibm.ccd.ui.dataobjects.workflow.Steps;
import com.ibm.ccd.ui.mediator.ConfigManager;
import com.ibm.ccd.ui.mediator.ISoapMessageKeys;
import com.ibm.ccd.ui.mediator.XMLHelper;
import com.ibm.ccd.ui.util.UIUtils;


/**
 *  Provides the functionality to generate the CatalogItem XMLs for
 *  various user actions like getting spec/details/create/update/delete/clone etc.
 *  Note: This comp. is not optimized for the XML generation, presently it uses the
 *        Stringbuffer, A better option could be making serialized dom elements and setting
 *        values using DOM.. but this may have little overhead in mem. usage.
 *        Need to re-look at the design again..
 */
public class CatalogItemGenerator {
    public static final String COMP_NAME = "CatalogItemGenerator";
    private static Stack groupStack = new Stack();

    /**
     *  Prepares the WPC specific XML request for getting the catalog specification.
     */
    public static String prepareCatalogSpecRequestXML(String parentCategory,
        String catalogID) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(parentCategory).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">")
                  .append(catalogID).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">");

        return requestXML.toString();
    }

    /**
             *  Prepares the WPC specific XML request for getting the catalog specification.
             */
    public static String prepareLookupTableSpecRequestXML(String lookupID) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">")
                  .append(lookupID).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">");

        return requestXML.toString();
    }

    /**
     *  Prepares the WPC specific XML request for getting the item details.
     */
    public static String prepareItemDetailsRequestXML(String parentCtgyID,
        String catalogID, String itemID) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(parentCtgyID).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATALOG_NAME).append("> ")
                  .append(catalogID).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.TRADE_ITEM).append(" ")
                  .append(ISoapMessageKeys.ID).append("=\"").append(itemID)
                  .append("\">").append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.TRADE_ITEM).append(">");

        return requestXML.toString();
    }

    /**
     *  Prepares the WPC specific XML request for getting the item clone.
     *  Note: The request xml is same as that of item details
     */
    public static String prepareItemCloneRequestXML(String parentCtgyID,
        String catalogID, String itemID) {
        return prepareItemDetailsRequestXML(parentCtgyID, catalogID, itemID);
    }

    /**
     *  Prepares the WPC specific XML request for creating the item
     */
    public static String prepareItemAddRequestXML(ICatalogItem item) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(item.getParentCategoryID())
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATALOG_NAME).append("> ")
                  .append(item.getCatalogID()).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">");

        if (item.getCollaborationArea() != null) {
            requestXML.append(getCollabrationAreaInfo(item));
        }

        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
				  .append(ISoapMessageKeys.TRADE_ITEM).append(" ")
				  .append(ISoapMessageKeys.ID).append("=\"")
				  .append(item.getPrimaryKeyValue()).append("\">")
                  .append(generateAttributesXML(item.getAttributes()))
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.TRADE_ITEM).append(">");

        return requestXML.toString();
    }

    /**
     *  Prepares the WPC specific XML request for updating the item
     */
    public static String prepareItemUpdateRequestXML(ICatalogItem item) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(item.getParentCategoryID())
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATALOG_NAME).append("> ")
                  .append(item.getCatalogID()).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">");

        if (item.getCollaborationArea() != null) {
            requestXML.append(getCollabrationAreaInfo(item));
        }

        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.TRADE_ITEM).append(" ")
                  .append(ISoapMessageKeys.ID).append("=\"")
                  .append(item.getItemID()).append("\">")
                  .append(generateAttributesXML(item.getAttributes()))
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.TRADE_ITEM).append(">");

        return requestXML.toString();
    }

    /**
     *  Prepares the WPC specific XML request for deleting the item
     */
    public static String prepareItemDeleteRequestXML(String parenetCategory,
        String catalogID, String itemID) {
        StringBuffer requestXML = new StringBuffer();
        requestXML.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(parenetCategory).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.PARENT_CATEGORY).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">  ")
                  .append(catalogID).append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.CATALOG_NAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.TRADE_ITEM).append("  ")
                  .append(ISoapMessageKeys.ID).append("=\"").append(itemID)
                  .append("\"/>");

        return requestXML.toString();
    }

    ////////////////////////////////////////////
    // Utility methods in generaing the xml ///
    ///////////////////////////////////////////

    /**
            *  Generates the attribute list xml  for the item.
            */
    protected static String generateAttributesXML(Attribute[] attributes) {
        if ((attributes == null) || (attributes.length == 0)) {
            return null;
        }

        StringBuffer stringBuffer = new StringBuffer(2048);
        stringBuffer.append(XMLHelper.WPC_NS_REF_BEG)
                    .append(ISoapMessageKeys.ATTRIBUTE_LIST).append(">");
        Map cacheTemplatesMap = null;
        
        String strAttribute = "";
        String strAttributeValues = "";
		String strAttributeValue = "";
		ICacheFactory cacheFactory = CacheFactory.getInstance();
		strAttribute = (String) cacheFactory.getCachedObject(
				CacheConstants.TEMPLATES, CacheConstants.TEMPLATE_ATTRIBUTE);

		strAttributeValue = (String) cacheFactory.getCachedObject(
				CacheConstants.TEMPLATES,
				CacheConstants.TEMPLATE_ATTRIBUTE_VALUE);

		strAttributeValues = (String) cacheFactory.getCachedObject(
				CacheConstants.TEMPLATES,
				CacheConstants.TEMPLATE_ATTRIBUTE_VALUES);
       
    	cacheTemplatesMap = new HashMap(3);
    	/*
		 * <wpc:attribute name="{0}" type="{1}">{2}</wpc:attribute>
		 */
		cacheTemplatesMap.put(CacheConstants.TEMPLATE_ATTRIBUTE,strAttribute);
		/*
		 * <wpc:values>{0}</wpc:values>
		 */
		cacheTemplatesMap.put(CacheConstants.TEMPLATE_ATTRIBUTE_VALUES,strAttributeValues);
		/*
		 * <wpc:value occurrence="{0}"><![CDATA[{1}]]></wpc:value>
		 */			
		cacheTemplatesMap.put(CacheConstants.TEMPLATE_ATTRIBUTE_VALUE,strAttributeValue);

        for (int i = 0; i < attributes.length; ++i) {
        	generateAttributeXML(attributes[i], stringBuffer, cacheTemplatesMap);
        }

        stringBuffer.append(XMLHelper.WPC_NS_REF_END)
                    .append(ISoapMessageKeys.ATTRIBUTE_LIST).append(">");

        return stringBuffer.toString();
    }

    /**
     * @param attr
     * @param xmlBuffer
     * @param cacheTemplatesMap
     */
    protected static void generateAttributeXML(Attribute attr,
        StringBuffer xmlBuffer, Map cacheTemplatesMap) {
		/**
		 * Changes to accomodate Attributes which are localized, and contain several locale values
		 */
		Attribute attribute = attr;

		if (attribute !=  null && attribute.isLocalized()) {
			for (int i = 0; i < attribute.getLocalizedValues().size(); i++) {
				generateAttributeXML((Attribute) attribute.getLocalizedValues()
														  .get(i), xmlBuffer,cacheTemplatesMap);
			}
		}
		
		if(!attr.hasAnyParentGroupMultivalued()) {
			if(!attr.isPrimaryKey() && !attr.isValueChanged())
				return;
		}

        if ( cacheTemplatesMap != null && getAttributeFromCache(attribute,xmlBuffer, cacheTemplatesMap)) {
			return;
		}
        switch (attribute.getType()) {
        case AttributeTypes.STRING: {
            StringAttribute stringAttribute = (StringAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = stringAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i], i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.INTEGER: {
            IntegerAttribute integerAttribute = (IntegerAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = integerAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i] + "", i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.NUMBER: {
            NumberAttribute numberAttribute = (NumberAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = numberAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i] + "", i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.DATE: {
            DateAttribute dateAttribute = (DateAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = dateAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i], i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.FLAG: {
            FlagAttribute flagAttribute = (FlagAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = flagAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i], i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.TIMEZONE: {
            TimeZoneAttribute timeZoneAttribute = (TimeZoneAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = timeZoneAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                	if (values[i] == null) {
                		NVPair[] nvPair = TimeZoneAttribute.getTimeZoneEnum();
                		values[i] = (String)nvPair[0].getName();
					}
            		addStringValue(values[i], i, xmlBuffer);                    
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.CURRENCY: {
            CurrencyAttribute currencyAttribute = (CurrencyAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = currencyAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i], i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.PASSWORD: {
            PasswordAttribute passwordAttribute = (PasswordAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = passwordAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i], i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }
		case AttributeTypes.PERIOD :
			// special element contains the nested attributes for the start and end periods
			{
				PeriodAttribute periodAttribute = (PeriodAttribute) attribute;

				xmlBuffer
					.append(XMLHelper.WPC_NS_REF_BEG)
					.append(ISoapMessageKeys.ATTRIBUTE)
					.append("  ")
					.append(ISoapMessageKeys.ATTRIB_NAME)
					.append("=\"")
					.append(attribute.getName())
					.append("\" ")
					.append(ISoapMessageKeys.ATTRIB_TYPE)
					.append("=\"")
					.append(AttributeTypes.getString(attribute.getType()))
					.append("\">")
					.append(XMLHelper.WPC_NS_REF_BEG)
					.append(ISoapMessageKeys.ATTRIB_VALUES)
					.append(">");

				for (int k = 0;
					k < periodAttribute.getValueList().size();
					k++) {
					xmlBuffer
						.append(XMLHelper.WPC_NS_REF_BEG)
						.append(ISoapMessageKeys.PERIODVALUE)
						.append("  ")
						.append(ISoapMessageKeys.ATTRIB_PERIOD_ENDDATE)
						.append("=\"")
						.append(periodAttribute.getEndDate(k))
						.append("\"  ")
						.append(ISoapMessageKeys.OCCURRENCE)
						.append("=\"")
						.append(k)
						.append("\"  ")
						.append(ISoapMessageKeys.ATTRIB_PERIOD_STARTDATE)
						.append("=\"")
						.append(periodAttribute.getStartDate(k))
						.append("\"/>");
				}

				xmlBuffer
					.append(XMLHelper.WPC_NS_REF_END)
					.append(ISoapMessageKeys.ATTRIB_VALUES)
					.append(">");

				xmlBuffer
					.append(XMLHelper.WPC_NS_REF_END)
					.append(ISoapMessageKeys.ATTRIBUTE)
					.append(">");
				return;
			}


        case AttributeTypes.SEQUENCE:// Not supported
            return;

        case AttributeTypes.RELATIONSHIP: {
            RelationshipAttribute relationshipAttribute = (RelationshipAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                NVPair[] values = relationshipAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    xmlBuffer.append(XMLHelper.WPC_NS_REF_BEG)
                             .append(ISoapMessageKeys.RELATION).append(" ")
                             .append(ISoapMessageKeys.RELATION_TO).append("=\"")
                             .append(values[i].getName()).append("\"  ")
                             .append(ISoapMessageKeys.OCCURRENCE).append("=\"")
                             .append(i).append("\">")
                             .append(values[i].getValue())
                             .append(XMLHelper.WPC_NS_REF_END)
                             .append(ISoapMessageKeys.RELATION).append(">");
                }
            } else {
                xmlBuffer.append(XMLHelper.WPC_NS_REF_BEG)
                         .append(ISoapMessageKeys.RELATION).append(">")
                         .append(XMLHelper.WPC_NS_REF_END)
                         .append(ISoapMessageKeys.RELATION).append("/>");
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.BINARY: {
            BinaryAttribute binaryAttribute = (BinaryAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                FileUploadResource[] values = binaryAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
					String value = getFileNameFromDocPath(values[i].getName());
                	addStringValue((value==null)?"":value, i, xmlBuffer);
                }
            } else {
            		addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.IMAGE: {
            ImageAttribute imageAttribute = (ImageAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                FileUploadResource[] values = imageAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
					String value = getFileNameFromDocPath(values[i].getName());
					addStringValue((value==null)?"":value, i, xmlBuffer);                	
                }
            } else {
                addEmptyFileValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.THUMB_NAIL_IMAGE: {
            ThumbnailImageAttribute thumbnailImageAttribute = (ThumbnailImageAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                FileUploadResource[] values = thumbnailImageAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
					String value = getFileNameFromDocPath(values[i].getName());
					addStringValue((value==null)?"":value, i, xmlBuffer);                	
                }
            } else {
                addEmptyFileValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.URL: {
            URLAttribute urlAttribute = (URLAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = urlAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    if (values[i] != null) {
                        addStringValue(values[i].toString(), i, xmlBuffer);
                    } else {
                        addStringValue("", i, xmlBuffer);
                    }
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.IMAGE_URL: {
            ImageURLAttribute imageURLAttribute = (ImageURLAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = imageURLAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
					if (values[i] != null) {
                    	addStringValue(values[i].toString(), i, xmlBuffer);
					}	else {
						addStringValue("", i, xmlBuffer);
					}
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.THUMB_NAIL_IMAGE_URL: {
            ThumbnailImageURLAttribute thumbnailImageURLAttribute = (ThumbnailImageURLAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = thumbnailImageURLAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                	if (values[i] != null) {
                    	addStringValue(values[i].toString(), i, xmlBuffer);
                	} else  {
                		addStringValue("", i, xmlBuffer);
                	}
                	
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.STRING_ENUMERATION: {
            StringEnumAttribute stringEnumAttribute = (StringEnumAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = stringEnumAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i].toString(), i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.NUMBER_ENUMERATION: {
            NumberEnumAttribute numberEnumAttribute = (NumberEnumAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = numberEnumAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i] + "", i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.LOOKUP_TABLE: {
            LookupTableAttribute lookupTableAttribute = (LookupTableAttribute) attribute;

            addAttributeHeader(attribute, xmlBuffer);

            if (attribute.getCurrentOccurrence() != 0) {
                String[] values = lookupTableAttribute.getValues();

                for (int i = 0; i < values.length; ++i) {
                    addStringValue(values[i], i, xmlBuffer);
                }
            } else {
                addEmptyStringValue(xmlBuffer);
            }

            addAttributeFooter(attribute, xmlBuffer);

            return;
        }

        case AttributeTypes.GROUP: {
            GroupAttribute groupAttribute = (GroupAttribute) attribute;
            groupAttribute.setGroupIndex(-1);
			groupStack.push(groupAttribute);

	            while(groupAttribute.hasNextGroup() ) {
	            	
	                Attribute[] attributes = groupAttribute.nextGroup();
	                
	                if(groupAttribute.getGroupIndex() == 0) {
						addGroupAttributeHeader(groupAttribute, xmlBuffer);
	                }else {
						addGroupAttributeNextHeader(groupAttribute,xmlBuffer);
	                }
						
	                for (int i = 0; i < attributes.length; i++) {
	                    generateAttributeXML(attributes[i], xmlBuffer, cacheTemplatesMap);
	                }
					
					if(!groupStack.isEmpty() && !groupAttribute.hasNextGroup()) {
						groupAttribute = (GroupAttribute)groupStack.pop();
						addGroupAttributeFooter(groupAttribute, xmlBuffer);						
 					}else{
						addGroupAttributeNextFooter(groupAttribute, xmlBuffer);
 					}
					
				}
        }
        
      } // end of switch	
    }
    /**
     * This method prepare xml for attribute from Cached templates
	 * Here are 3 templates for attribute xml creation used in cache
	 *  1. <wpc:attribute name="{0}" type="{1}">{2}</wpc:attribute>
	 *  2. <wpc:values>{0}</wpc:values>
	 * 	3. <wpc:value occurence="{2}" ><![CDATA[{3}]]></wpc:value>
	 *  All templates stored in Map
     * @param cacheTemplatesMap Map
     * @param attribute Attribute  
     * @param xmlBuffer StringBuffer
	 */
	private static boolean getAttributeFromCache(Attribute attribute,
			StringBuffer xmlBuffer, Map cacheTemplatesMap) {
		try {
			if (attribute.getType() == AttributeTypes.PERIOD || 
					attribute.getType() == AttributeTypes.RELATIONSHIP || 
					attribute.getType() == AttributeTypes.UNSUPPORTED ||
					attribute.getType() == AttributeTypes.LOOKUP_TABLE ||
					attribute.getType() == AttributeTypes.SEQUENCE) {
				return false;
			}
			String strAttribute = (String)cacheTemplatesMap.get(CacheConstants.TEMPLATE_ATTRIBUTE);
			String strAttributeValue = (String)cacheTemplatesMap.get(CacheConstants.TEMPLATE_ATTRIBUTE_VALUE);
			String strAttributeValues = (String)cacheTemplatesMap.get(CacheConstants.TEMPLATE_ATTRIBUTE_VALUES);
			
			strAttributeValue = getValueXMLForCache(attribute,strAttributeValue);

			if (strAttributeValue == null) {
				return false;
			}
			/*
			 * This replaces {0} in <wpc:values>{0} </wpc:values> with <value
			 * ..>(strAttributeValue String) list for all occurence
			 */
			String[] replacementsArrForValues = { strAttributeValue };
			strAttributeValues = MessageFormat.format(strAttributeValues,
					replacementsArrForValues);
			/*
			 * This replaces name,type and values in <wpc:attribute name="{0}" type="{1}">{2} </wpc:attribute>
			 */			
			String[] replacementsArrForAttribute = { attribute.getName(),
					AttributeTypes.getString(attribute.getType()),strAttributeValues };
			String completeAttributeString = MessageFormat.format(strAttribute,
					replacementsArrForAttribute);
			
			xmlBuffer.append(completeAttributeString);
		
		} catch(Exception ex){
			ex.printStackTrace();
			return false;
		}
		return true;
	}
	/**
	 * getValueXMLForCache: This replaces the <value> template with data and returns 
	 * generated string
	 * @param attribute
	 * @param attributeValueTemplate
	 * @return
	 */
	private static String getValueXMLForCache(Attribute attribute,
			String attributeValueTemplate) {
		StringBuffer sb = new StringBuffer();

		switch (attribute.getType()) {
		case AttributeTypes.STRING: {
			StringAttribute convAttribute = (StringAttribute) attribute;

			String[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String[] valueString = { i + "", values[i] + "" };
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.INTEGER: {
			IntegerAttribute convAttribute = (IntegerAttribute) attribute;
			String[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String[] valueString = { i + "", values[i] + "" };
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.NUMBER: {
			NumberAttribute convAttribute = (NumberAttribute) attribute;
			String[] values = convAttribute.getValues();

			for (int i = 0; i < values.length; ++i) {
				String[] valueString = { i + "", values[i] + "" };
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.BINARY: {
			BinaryAttribute convAttribute = (BinaryAttribute) attribute;
			FileUploadResource[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String value = getFileNameFromDocPath(values[i].getName());
				String[] valueString = { i + "",(value == null)?"":value};
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.IMAGE: {
			ImageAttribute convAttribute = (ImageAttribute) attribute;
			FileUploadResource[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String value = getFileNameFromDocPath(values[i].getName());
				String[] valueString = { i + "",(value == null)?"":value};
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.THUMB_NAIL_IMAGE: {
			ThumbnailImageAttribute convAttribute = (ThumbnailImageAttribute) attribute;

			FileUploadResource[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String value = getFileNameFromDocPath(values[i].getName());
				String[] valueString = { i + "",(value == null)?"":value};
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.URL: {
			URLAttribute convAttribute = (URLAttribute) attribute;
			String[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String temp = values[i]==null?"":values[i]+"";
				String[] valueString = { i + "",  temp };
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.IMAGE_URL: {
			ImageURLAttribute convAttribute = (ImageURLAttribute) attribute;
			String[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String temp = values[i]==null?"":values[i]+"";
				String[] valueString = { i + "", temp };
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.THUMB_NAIL_IMAGE_URL: {
			ThumbnailImageURLAttribute convAttribute = (ThumbnailImageURLAttribute) attribute;
			String[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String temp = values[i]==null?"":values[i]+"";
				String[] valueString = { i + "", temp };
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.STRING_ENUMERATION: {
			StringEnumAttribute convAttribute = (StringEnumAttribute) attribute;
			String[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String[] valueString = { i + "", values[i] + "" };
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}

		case AttributeTypes.NUMBER_ENUMERATION: {
			NumberEnumAttribute convAttribute = (NumberEnumAttribute) attribute;

			String[] values = convAttribute.getValues();
			for (int i = 0; i < values.length; ++i) {
				String[] valueString = { i + "", values[i] + "" };
				sb.append(MessageFormat.format(attributeValueTemplate,
						valueString));
			}
			return sb.toString();
		}
		default:
			return null;
		} // end of switch
	}

    private static void addGroupAttributeHeader(Attribute attribute,
        StringBuffer xmlBuffer) {
        xmlBuffer.append(XMLHelper.WPC_NS_REF_BEG);

        if (attribute.isSubAttribute()) {
            xmlBuffer.append(ISoapMessageKeys.SUB_ATTRIBUTE);
        } else {
            xmlBuffer.append(ISoapMessageKeys.ATTRIBUTE);
        }

        xmlBuffer.append("  ").append(ISoapMessageKeys.ATTRIB_NAME).append("=\"")
                 .append(attribute.getName()).append("\" ")
                 .append(ISoapMessageKeys.ATTRIB_TYPE).append("=\"")
                 .append(AttributeTypes.getString(attribute.getType()))
                 .append("\">").append(XMLHelper.WPC_NS_REF_BEG)
                 .append(ISoapMessageKeys.ATTRIB_VALUES).append(">")
                 .append(XMLHelper.WPC_NS_REF_BEG)
                 .append(ISoapMessageKeys.GROUP_ATTTRIBUTE+" ")
                 .append(ISoapMessageKeys.OCCURRENCE + "=\"")
                 .append(((GroupAttribute)attribute).getGroupIndex() +"\"")
                 .append(">");
    }

    private static void addAttributeHeader(Attribute attribute,
        StringBuffer xmlBuffer) {
        xmlBuffer.append(XMLHelper.WPC_NS_REF_BEG);

        if (attribute.isSubAttribute()) {
            xmlBuffer.append(ISoapMessageKeys.SUB_ATTRIBUTE);
        } else {
            xmlBuffer.append(ISoapMessageKeys.ATTRIBUTE);
        }

        xmlBuffer.append("  ").append(ISoapMessageKeys.ATTRIB_NAME).append("=\"")
                 .append(attribute.getName()).append("\" ")
                 .append(ISoapMessageKeys.ATTRIB_TYPE).append("=\"")
                 .append(AttributeTypes.getString(attribute.getType()))
                 .append("\">").append(XMLHelper.WPC_NS_REF_BEG)
                 .append(ISoapMessageKeys.ATTRIB_VALUES).append(">");
    }

    private static void addStringValue(String value, int occurrence,
        StringBuffer xmlBuffer) {
        xmlBuffer.append(XMLHelper.WPC_NS_REF_BEG)
                 .append(ISoapMessageKeys.ATTRIB_VALUE).append("  ")
                 .append(ISoapMessageKeys.OCCURRENCE).append("=\"")
                 .append(occurrence).append("\">")
	 			 .append("<![CDATA[")
	 			 .append(value)
				 .append("]]>")
                 .append(XMLHelper.WPC_NS_REF_END)
                 .append(ISoapMessageKeys.ATTRIB_VALUE).append(">");
    }

    private static void addFileValue(FileUploadResource value, int occurrence,
        StringBuffer xmlBuffer) {
        xmlBuffer.append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.FILE)
                 .append(" ").append(ISoapMessageKeys.TYPE).append("=\"")
                 .append(value.getType()).append("\" ")
                 .append(ISoapMessageKeys.OCCURRENCE).append("=\"")
                 .append(occurrence).append("\">")
                 .append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.NAME)
                 .append(">").append(getFileNameFromDocPath(value.getName()))
                 .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.NAME)
                 .append(">").append(XMLHelper.WPC_NS_REF_BEG)
                 .append(ISoapMessageKeys.DIRECTORY).append(">")
                 .append(value.getDirectory()).append(XMLHelper.WPC_NS_REF_END)
                 .append(ISoapMessageKeys.DIRECTORY).append(">")
                 .append(XMLHelper.WPC_NS_REF_BEG)
                 .append(ISoapMessageKeys.SERVERNAME).append(">")
                 .append(value.getServer()).append(XMLHelper.WPC_NS_REF_END)
                 .append(ISoapMessageKeys.SERVERNAME).append(">")
                 .append(XMLHelper.WPC_NS_REF_BEG)
                 .append(ISoapMessageKeys.USERID).append(">")
                 .append(value.getUserID()).append(XMLHelper.WPC_NS_REF_END)
                 .append(ISoapMessageKeys.USERID).append(">")
                 .append(XMLHelper.WPC_NS_REF_BEG)
                 .append(ISoapMessageKeys.PASSWORD).append(">")
                 .append(value.getPassword()).append(XMLHelper.WPC_NS_REF_END)
                 .append(ISoapMessageKeys.PASSWORD).append(">")
                 .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.FILE)
                 .append(">");
    }

    private static void addEmptyFileValue(StringBuffer xmlBuffer) {
        xmlBuffer.append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.FILE)
                 .append(">").append(XMLHelper.WPC_NS_REF_BEG)
                 .append(ISoapMessageKeys.NAME).append("/>")
                 .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.FILE)
                 .append(">");
    }

    private static void addEmptyStringValue(StringBuffer xmlBuffer) {
        xmlBuffer.append(XMLHelper.WPC_NS_REF_BEG) 
                 .append(ISoapMessageKeys.ATTRIB_VALUE)
                 .append(" ").append(ISoapMessageKeys.OCCURRENCE)
                 .append("=\"0").append("\"/>");
    }

    private static void addAttributeFooter(Attribute attribute,
        StringBuffer xmlBuffer) {
        xmlBuffer.append(XMLHelper.WPC_NS_REF_END)
                 .append(ISoapMessageKeys.ATTRIB_VALUES).append(">").append(XMLHelper.WPC_NS_REF_END);

        if (attribute.isSubAttribute()) {
            xmlBuffer.append(ISoapMessageKeys.SUB_ATTRIBUTE);
        } else {
            xmlBuffer.append(ISoapMessageKeys.ATTRIBUTE);
        }

        xmlBuffer.append(">");
    }

	private static void addGroupAttributeNextHeader(GroupAttribute attribute, StringBuffer xmlBuffer) {
		xmlBuffer.append(XMLHelper.WPC_NS_REF_BEG)
				.append(ISoapMessageKeys.GROUP_ATTTRIBUTE )
				.append(" " + ISoapMessageKeys.OCCURRENCE+"=\"")
				.append(attribute.getGroupIndex())
				.append("\">");
		//xmlBuffer.append("<wpc:groupvalue occurrence=\"" + attribute.getGroupIndex() +"\">");
	}
	
	private static void addGroupAttributeNextFooter(GroupAttribute attribute, StringBuffer xmlBuffer) {
		xmlBuffer.append(XMLHelper.WPC_NS_REF_END)
				.append(ISoapMessageKeys.GROUP_ATTTRIBUTE)
				.append(">");
		//xmlBuffer.append("</wpc:groupvalue>");
	}
	
    private static void addGroupAttributeFooter(Attribute attribute,
        StringBuffer xmlBuffer) {
        xmlBuffer.append(XMLHelper.WPC_NS_REF_END)
                 .append(ISoapMessageKeys.GROUP_ATTTRIBUTE).append(">")
                 .append(XMLHelper.WPC_NS_REF_END)
                 .append(ISoapMessageKeys.ATTRIB_VALUES).append(">").append(XMLHelper.WPC_NS_REF_END);

        if (attribute.isSubAttribute()) {
            xmlBuffer.append(ISoapMessageKeys.SUB_ATTRIBUTE);
        } else {
            xmlBuffer.append(ISoapMessageKeys.ATTRIBUTE);
        }

        xmlBuffer.append(">");
    }

    private static String getFileNameFromDocPath(String docPath) {
        int index = -1;

        if (docPath != null) {
            index = docPath.lastIndexOf(File.pathSeparator);

            if (index != -1) {
                return docPath.substring(index + 1);
            } else {
                index = docPath.lastIndexOf('/');

                // check for Unix Path Separator
                if (index != -1) {
                    return docPath.substring(index + 1);
                }
            }
        }

        return docPath;
    }

    private static String getCollabrationAreaInfo(ICatalogItem item) {
        CollaborationArea collabArea = item.getCollaborationArea();
        StringBuffer buffer = new StringBuffer();

        if (collabArea != null) {
            buffer.append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_COLL_AREA).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.NAME)
                  .append(">").append(collabArea.getCollabName())
                  .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.NAME)
                  .append(">").append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_NAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG).append(ISoapMessageKeys.NAME)
                  .append(">").append(collabArea.getWorkflowName())
                  .append(XMLHelper.WPC_NS_REF_END).append(ISoapMessageKeys.NAME)
                  .append(">").append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_STEP).append(">")
                  .append(XMLHelper.WPC_NS_REF_BEG)
                  .append(ISoapMessageKeys.WORKFLOW_STEPNAME).append(">")
                  .append(((Steps) collabArea.getSteps().get(0)).getStepName())
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_STEPNAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_STEP).append(">")
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_NAME).append(">")
                  .append(XMLHelper.WPC_NS_REF_END)
                  .append(ISoapMessageKeys.WORKFLOW_COLL_AREA).append(">")
                  .append("");

            return buffer.toString();
        }

        return "";
    }
	
	/**
	 * replaces the HTML "&lt;" as "<" and "&gt;" as ">"
	 * @param str
	 * @return
	 */
	private static String replaceHTMLChar(String str) {
		if(UIUtils.isEmpty(str)) return str;
    	
		return str.replaceAll("&lt;","<").replaceAll("&gt;",">");
	}
}
