
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.mediator;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import javax.servlet.ServletContext;

import org.w3c.dom.Element;

import com.ibm.ccd.ui.cache.CacheConstants;
import com.ibm.ccd.ui.cache.CacheManager;
import com.ibm.ccd.ui.cache.CacheNotEnabledException;
import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.util.IReference;


/**
 * Represents the Config Information related to the Mediator and others.
 * Presently the support is added to parse and keep the SoapActionURLs,
 * Portlet Actions to pages mapping information
 * and Timezones exported from WPC.
 *
 * Note: In order to use this , it must be intialized by the ServletContext.
 * typically this is done by the MediatorInitializer Servlet.
 */
public class ConfigManager {
    public static final String COMP_NAME = "ConfigManager";

    // singleton instance of the configManager
    private static ConfigManager configManager;

    // config file for the timezones.
    private static final String TIME_ZONE_CONF = "/WEB-INF/config/timezonevalues.xml";

    // config file for the soap actions.
    private static final String WPC_SOAP_CONFIG = "/WEB-INF/config/soapactionurls.xml";

    // config file for the portlet actions to pages mapping
    private static final String WPC_PORTLETACTION_TO_PAGE_CONFIG = "/WEB-INF/config/portletActions2pages.xml";

    //	Constants representing the tags which need to be parsed in the action to page mappings config file
    public static final String WPC_PORTLETACTION_TO_PAGE_ELEMENT_TAG = "action2page";
    public static final String WPC_PORTLETACTION_TO_PAGE_ACTION_TAG = "action";

    // configuration file to identify the framework parameters to work with
    private static final String WPC_FRAMEWORK_CONFIG = "/WEB-INF/config/framework_configuration.xml";
    
    // configuration file to check enable and disable transactions
    private static final String WPC_LDAP_CONFIG = "/WEB-INF/config/ldap_config.properties";
    private static final String WPC_XML_TEMPLATE_DIR = "/WEB-INF/config/templates";
    
    public static final String WPC_PROPERTY_ELEMENT_TAG = "property";
    public static final String WPC_PROPERTY_NAME_ELEMENT_TAG = "name";
    private static String contextRootPath = "";

    // timeZone values
    private NVPair[] timeZones;

    // soapAction URLs 
    private HashMap soapActionURLs;

    // action to page mapping
    private HashMap actionToPageMappings;
    private HashMap frameworkConfigMappings;
    private Properties ldapProp ;
    private HashMap cacheConfigTemplates;
    /**
     * Private constructor.
     */
    private ConfigManager() {
    }

    /**
     * Returns the ConfigManager instance
     */
    public static ConfigManager getInstance() {
        if (configManager == null) {
            synchronized (ConfigManager.class) {
                if (configManager == null) {
                    configManager = new ConfigManager();
                }
            }
        }

        return configManager;
    }

    /**
     * Initalizes by resolving the realpath from the servlet config.
     * @see javax.servlet.ServletContext
     */
    public void init(ServletContext servletContext) {
        //	(1)...	Load the time zones		
        prepareTimeZones(servletContext.getRealPath(TIME_ZONE_CONF));

        //	(2)...	Load the SOAP Actions
        prepareSoapActionURLs(servletContext.getRealPath(WPC_SOAP_CONFIG));

        //	(3)...	Load the Actions to Page mappings
        prepareActionToPageMappings(servletContext.getRealPath(
                WPC_PORTLETACTION_TO_PAGE_CONFIG));

        //  (4)... Load the frameworkConfig that this entire application will work with
        prepareFrameworkConfiguration(servletContext.getRealPath(
                WPC_FRAMEWORK_CONFIG));
        //  (5)... Load the ldap Config 
        prepareLDAPConfig(servletContext.getRealPath(
                WPC_LDAP_CONFIG));
        // (5)... Load the template xmls that this entire application will work with
        cacheXMLTemplates(servletContext.getRealPath(WPC_XML_TEMPLATE_DIR));
        
        contextRootPath = servletContext.getRealPath("/");
    }

	/**
	 * @param templateDir
	 */
	public void cacheXMLTemplates(String templateDir) {
		CacheManager cacheManager = null;
		
		if (templateDir == null) {
			templateDir = contextRootPath + WPC_XML_TEMPLATE_DIR;
		}
        
		if (IReference.COND_CMPL) {
            System.out.println(
                "ConfigManager.cachteXMLTemplates() :- Loading the templates file :");
        }
		try {
		cacheManager = CacheManager.getInstance();
		} catch (CacheNotEnabledException e) {
			e.printStackTrace();
		}

			String strSoapHeaderXML = XMLHelper.readFile(templateDir
					+ File.separator + CacheConstants.TEMPLATE_SOAP_HEADER
					+ ".xml");

			//	Caching the Base template with place holders for occurence, Name
			//  and data <attribute>
			String strAttributeSTRINGXML = XMLHelper.readFile(templateDir
					+ File.separator + CacheConstants.TEMPLATE_ATTRIBUTE
					+ ".xml");

			//	Caching the template with place holders for <values>..
			String strAttributeValuesSTRINGXML = XMLHelper.readFile(templateDir
					+ File.separator + CacheConstants.TEMPLATE_ATTRIBUTE_VALUES
					+ ".xml");
			//	Caching the Base template with place holders for <value>..

			String strAttributeValueSTRINGXML = XMLHelper.readFile(templateDir
					+ File.separator + CacheConstants.TEMPLATE_ATTRIBUTE_VALUE
					+ ".xml");
			
			if (cacheManager != null && CacheManager.isCacheEnabled()) {
			cacheManager.putInCache(CacheConstants.TEMPLATES,
					CacheConstants.TEMPLATE_SOAP_HEADER, strSoapHeaderXML);

			cacheManager.putInCache(CacheConstants.TEMPLATES,
					CacheConstants.TEMPLATE_ATTRIBUTE, strAttributeSTRINGXML);

			cacheManager.putInCache(CacheConstants.TEMPLATES,
					CacheConstants.TEMPLATE_ATTRIBUTE_VALUES,
					strAttributeValuesSTRINGXML);

			cacheManager.putInCache(CacheConstants.TEMPLATES,
					CacheConstants.TEMPLATE_ATTRIBUTE_VALUE,
					strAttributeValueSTRINGXML);
			
			} else {
				cacheConfigTemplates = new HashMap(6);
				cacheConfigTemplates.put(CacheConstants.TEMPLATE_SOAP_HEADER,
						strSoapHeaderXML);
				cacheConfigTemplates.put(CacheConstants.TEMPLATE_ATTRIBUTE,
						strAttributeSTRINGXML);
				cacheConfigTemplates.put(CacheConstants.TEMPLATE_ATTRIBUTE_VALUES,
						strAttributeValuesSTRINGXML);
				cacheConfigTemplates.put(CacheConstants.TEMPLATE_ATTRIBUTE_VALUE,
						strAttributeValueSTRINGXML);
			}

	}
    /**
     * Parses the timezone config file and prepares the
     * timeZone values (consumed by the TimeZoneAttribute)
     * @throws RuntimeException in case fails to parse the file
     * @see com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute
     */
    private void prepareTimeZones(String confFile) {
        if (IReference.COND_CMPL) {
            System.out.println(
                "ConfigManager.prepareTimeZones() :- Loading the config file :" +
                confFile);
        }

        Element timeZonesRootElement = XMLHelper.getRootElementFromFile(confFile);

        if (timeZonesRootElement == null) {
            System.out.println(
                "ERROR @ ConfigManager.prepareTimeZones() :- Not able to load the config file :" +
                confFile);

            return;
        }

        Element[] timeZoneElements = XMLHelper.getElements(timeZonesRootElement,
                "option");

        timeZones = new NVPair[timeZoneElements.length];

        for (int i = 0; i < timeZones.length; ++i) {
            timeZones[i] = new NVPair(XMLHelper.getAttributeValue(
                        timeZoneElements[i], "value"),
                    XMLHelper.getElementValue(timeZoneElements[i]));
        }

        if (IReference.COND_CMPL) {
            System.out.println(
                "ConfigManager.prepareTimeZones() :-  TimeZones  loaded successfully");
        }
    }
    /**
     * Parses the timezone config file and prepares the
     * timeZone values (consumed by the TimeZoneAttribute)
     * @throws RuntimeException in case fails to parse the file
     * @see com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute
     */
    private void prepareLDAPConfig(String confFile) {
        if (IReference.COND_CMPL) {
            System.out.println(
                "ConfigManager.prepareLDAPConfig() :- Loading the config file :" +
                confFile);
        }

       ldapProp = new Properties();
       InputStream instream = null;
       try {
		    instream = new FileInputStream(confFile);
		    ldapProp.load(instream);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} 


        if (ldapProp == null) {
            System.out.println(
                "ERROR @ ConfigManager.prepareLDAPConfig() :- Not able to load the config file :" +
                confFile);
            return;
        }
        
    }
    /**
     * Parses the soap-action-urls config file and prepares the
     * action-URL valuesMap (consumed by the WSHelper)
     * @throws RuntimeException in case fails to parse the file
     * @see WSHelper
     */
    private void prepareSoapActionURLs(String confFile) {
        if (IReference.COND_CMPL) {
            System.out.println(
                "ConfigManager.prepareSoapActionURLs() :- Loadin the config file :" +
                confFile);
        }

        Element soapActionURLRootElement = XMLHelper.getRootElementFromFile(confFile);

        if (soapActionURLRootElement == null) {
            System.out.println(
                "ERROR @ ConfigManager.prepareSoapActionURLs() :- Not able to load the config file :" +
                confFile);

            return;
        }

        Element[] soapActionURLElements = XMLHelper.getElements(soapActionURLRootElement,
                "soap-url");

        soapActionURLs = new HashMap(soapActionURLElements.length);

        for (int i = 0; i < soapActionURLElements.length; ++i) {
            soapActionURLs.put(XMLHelper.getAttributeValue(
                    soapActionURLElements[i], "action"),
                XMLHelper.getElementValue(soapActionURLElements[i]));
        }

        if (IReference.COND_CMPL) {
            Iterator keyIterator = soapActionURLs.keySet().iterator();

            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println(key + " : " + soapActionURLs.get(key));
            }

            System.out.println(
                "ConfigManager.prepareSoapActionURLs() :-  TimeZones  loaded successfully");
        }
    }

    /**
             * Parses the action-page mappings config file and prepares the
             * action-URL valuesMap (consumed by the Portlets)
             * @throws RuntimeException in case fails to parse the file
             */
    private void prepareActionToPageMappings(String confFile) {
        if (IReference.COND_CMPL) {
            System.out.println(
                "ConfigManager.prepareActionMaps() :- Loadin the config file :" +
                confFile);
        }

        Element actionMapRootElement = XMLHelper.getRootElementFromFile(confFile);

        if (actionMapRootElement == null) {
            System.out.println(
                "ERROR @ ConfigManager.prepareActionToPageMappings() :- Not able to load the config file :" +
                confFile);

            return;
        }

        Element[] actionMapElements = XMLHelper.getElements(actionMapRootElement,
                WPC_PORTLETACTION_TO_PAGE_ELEMENT_TAG);

        actionToPageMappings = new HashMap(actionMapElements.length);

        for (int i = 0; i < actionMapElements.length; ++i) {
            actionToPageMappings.put(XMLHelper.getAttributeValue(
                    actionMapElements[i], WPC_PORTLETACTION_TO_PAGE_ACTION_TAG),
                XMLHelper.getElementValue(actionMapElements[i]));
        }

        if (IReference.COND_CMPL) {
            Iterator keyIterator = actionToPageMappings.keySet().iterator();

            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println(key + " : " + actionToPageMappings.get(key));
            }

            System.out.println(
                "ConfigManager.prepareActionMaps() :-  Portlet Action to Page Mappings  loaded successfully");
        }
    }

    /**
                     * Parses the frameworkConfig config file and prepares the
                     * frameworkConfig list.
                     * @throws RuntimeException in case fails to parse the file
                     */
    private void prepareFrameworkConfiguration(String confFile) {
        if (IReference.COND_CMPL) {
            System.out.println(
                "ConfigManager.prepareFrameworkConfig() :- Loadin the config file :" +
                confFile);
        }

        Element frameworkConfigRootElement = XMLHelper.getRootElementFromFile(confFile);

        if (frameworkConfigRootElement == null) {
            System.out.println(
                "ERROR @ ConfigManager.prepareFrameworkConfig() :- Not able to load the config file :" +
                confFile);

            return;
        }

        Element[] frameworkConfigElements = XMLHelper.getElements(frameworkConfigRootElement,
                WPC_PROPERTY_ELEMENT_TAG);

        frameworkConfigMappings = new HashMap(frameworkConfigElements.length);

        for (int i = 0; i < frameworkConfigElements.length; ++i) {
            frameworkConfigMappings.put(XMLHelper.getAttributeValue(
                    frameworkConfigElements[i], WPC_PROPERTY_NAME_ELEMENT_TAG),
                XMLHelper.getElementValue(frameworkConfigElements[i]));
        }

        if (IReference.COND_CMPL) {
            Iterator keyIterator = frameworkConfigMappings.keySet().iterator();

            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println(key + " : " +
                    frameworkConfigMappings.get(key));
            }

            System.out.println(
                "ConfigManager.prepareFrameworkConfig() :-  Portlet Action to Page Mappings  loaded successfully");
        }
    }

    /**
     * @return SoapAction-URL Map
     */
    public HashMap getSoapActionURLs() {
        return soapActionURLs;
    }

    /**
     * @return TimeZone as an NVPair Array
     * @see com.ibm.ccd.ui.dataobjects.NVPair
     */
    public NVPair[] getTimeZones() {
        return timeZones;
    }
	public HashMap getCacheConfigTemplates() {
		return cacheConfigTemplates;
	}
	/**
	 * @param cacheConfigTemplates The cacheConfigTemplates to set.
	 */
	public void setCacheConfigTemplates(HashMap cacheConfigTemplates) {
		this.cacheConfigTemplates = cacheConfigTemplates;
	}

    /**
     * @return PortletAction to Page Map
     */
    public HashMap getPortletActionToPageMappings() {
        return actionToPageMappings;
    }

    /**
     * @return List of configured FrameworkConfig
     * Currently this supports just one frameworkConfig.
     */
    public HashMap getFrameworkConfig() {
        return frameworkConfigMappings;
    }
	/**
	 * @return Returns the ldapProp.
	 */
	public Properties getLdapProp() {
		return ldapProp;
	}
	public boolean isLdapEnabled() {
		String enable = "true";
		if (ldapProp != null) {
			enable = (String)ldapProp.getProperty("LDAP_ENABLED");
		}
		if (enable != null && enable.equalsIgnoreCase("false")) {
			return false;
		}
		return true;
	}
	/**
	 * @param ldapProp The ldapProp to set.
	 */
	public void setLdapProp(Properties ldapProp) {
		this.ldapProp = ldapProp;
	}
}
