
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package com.ibm.ccd.ui.mediator;

import java.util.List;
import java.util.Map;

import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.Attachment;
import com.ibm.ccd.ui.dataobjects.CategoryHierarchy;
import com.ibm.ccd.ui.dataobjects.FileDownloadResource;
import com.ibm.ccd.ui.dataobjects.Node;
import com.ibm.ccd.ui.dataobjects.items.ICatalogItem;
import com.ibm.ccd.ui.dataobjects.view.CatalogViewMaster;
import com.ibm.ccd.ui.dataobjects.workflow.WorkFlowTO;
import com.ibm.ccd.ui.dataobjects.workflow.Workflow;
import com.ibm.ccd.ui.portlets.search.core.SearchBean;

/**
 *        Represents the business functions available on the WPC.
 *  Provides the summary of the soap-calls supported by the mediator.
 *  Note: Most of the calls treats the Async. Process as the Application error
 *        and throws the WPCMediatorExcpetion setting APPError with the notificationBean.
 */
public interface IWPCMediator {
    /**
     * returns the first level of Hierarychy for the Catalogs
     * @param WPSContext context
     * @return CategoryHeirarychy - array of hierarychy names and which catalog it belongs
     * @throws WPCMediatorException in case of retrieval
     */
    public abstract CategoryHierarchy[] getCatalogs(WPSContext context)
        throws WPCMediatorException;

    /**
     *  returns the children of the catalog node,
     *  the  default invokation mode is Sync.
     *  @param parentID - parentNode ID of type String
     *  @return Node[] - representing the catalog node's first level children.
     *  @throws WPCMediatorException in case of retrieval errors
     */
    public abstract Node[] getCatalogChildren(String parentID,
        WPSContext context) throws WPCMediatorException;

    /**
     * returns the children of the catalog node for a particular parent
     * category, Hierarychy, Catalog
     * @param parentCategory
     * @param catalogName
     * @param hierarchyName
     * @param heirarchyType
     * @param context
     * @return
     * @throws WPCMediatorException
     */
    public abstract Node[] getCatalogChildren(String parentCategory,
        String catalogName, String hierarchyName, String heirarchyType,
        String childrenSize, WPSContext context) throws WPCMediatorException;

    /**
     *  returns the catalog Spec. from WPC for the given catalogID and parent CategoryID ,
     *  the  default invokation mode is Sync.
     *  @param parentCgyID - Parent CategoryID of the catalog
     *  @param catalogID - category ID of the catalog
     *  @return ICatalogItem - representing the catalogSpec.
     *  @throws WPCMediatorException in case of retrieval errors
     */
    public abstract ICatalogItem getCatalogItem(String parentCgyID,
        String catalogID, WPSContext context) throws WPCMediatorException;

    /**
      *  returns the catalog item from WPC for the given catalogID and parent CategoryID,
      *  the default invokation mode is Sync.
      *  @param parentCgyID - Parent CategoryID of the catalog
      *  @param catalogID - category ID of the catalog
      *  @param itemID - itemID (primaryKey)
      *  @return ICatalogItem
      *  @throws WPCMediatorException in case of retrieval errors
     */
    public abstract ICatalogItem getCatalogItem(String parentCgyID,
        String catalogID, String itemID, WPSContext context)
        throws WPCMediatorException;

	/**
	 * returs a list of CatalogViews for a given CatalogName
	 * @param catalogName
	 * @param context
	 * @return
	 * @throws WPCMediatorException
	 */
	public abstract List getCatalogViewList(
		String catalogName,
		WPSContext context)
		throws WPCMediatorException;

	/**
	 * 
	 * @param catalogNmae
	 * @param catalogViewName
	 * @param context
	 * @return
	 * @throws WPCMediatorException
	 */
	public abstract CatalogViewMaster getCatalogView(
		String catalogName,
		String catalogViewName,
		WPSContext context)
		throws WPCMediatorException;


    /**
     *  creates the item in WPC,the default invokation mode is Sync.
     *  @param item - of type ICatalogItem representing the item to create
     *  @return ItemID created(primarykey)
     *  @throws WPCMediatorException in case of creation errors/async. process
     */
    public abstract String createItem(ICatalogItem item, WPSContext context)
        throws WPCMediatorException;

    /**
     *  updates the item in WPC, the default invokation mode is Sync.
     *  @param item - of type ICatalogItem representing the item to create
     *  @throws WPCMediatorException in case of updation errors/ async. process.
     */
    public abstract void updateItem(ICatalogItem item, WPSContext context)
        throws WPCMediatorException;

    /**
     *  deletes the item in WPC, the default invokation mode is Sync.
     *  @param item - of type ICatalogItem representing the item to create
     *  @throws WPCMediatorException in case of updation errors/ async. process.
     */
    public abstract void deleteItem(ICatalogItem item, WPSContext context)
        throws WPCMediatorException;

    /**
     *  deletes the Item from WPC for the given itemID, catalogID and ParenetCategory ID
     *  The default invokation mode is Sync.
     *  @param parentCgyID - Parent CategoryID of the catalog
     *  @param catalogID - category ID of the catalog
     *  @param itemID - itemID (primaryKey)
     *  @throws WPCMediatorException in case of updation errors/ async. process.
     */
    public abstract void deleteItem(String parentCgy, String catalogID,
        String itemID, WPSContext context) throws WPCMediatorException;

    /**
     * Returns Clone of the CatalogItem  for the given  itemID,catalogID and parentCategory
     */
    public ICatalogItem getCatalogItemClone(String parentCgyID,
        String catalogID, String itemID, WPSContext context)
        throws WPCMediatorException;

    /**
     * Returns Clone of the CatalogItem  for the given  item
     */
    public ICatalogItem getCatalogItemClone(ICatalogItem item,
        WPSContext context) throws WPCMediatorException;

    /**
     *  uploads the file into WPC, and returns with the doc.path
     *  The default invokation mode is ASync.
     *  @param attachment - file to upload as an Attachment type
     *  @return  document path where the file has been uploaded. of type String
     *  @throws WPCMediatorException in case of updation errors/ async. process.
     *  @see com.ibm.ccd.ui.dataobjects.Attachment
     */
    public abstract String uploadFile(Attachment attachment, WPSContext context)
        throws WPCMediatorException;

    /**
     *  Provides the followup process action for the uplaod which triggers the
     *  for an update in wpc
     *  @param documentPath - path to the file to be updated in the WPC
     *  @return   transaction ID.
     *  @throws WPCMediatorException in case of updation errors/ async. process.
     */
    public abstract String processUploadedFile(String documentPath,
        String collabName, WPSContext context) throws WPCMediatorException;

    /**
     * Triggers the WPC to Perform the search based on the criteria and prepares the result
     * @param searchBean - provides the searchcritera and filled with the search results
     * @throws WPCMediatorException in case of updation errors/ async. process.
     */
    public abstract void performSearch(SearchBean searchBean, WPSContext context)
        throws WPCMediatorException;

    /**
     * Triggers the WPC to get the trasactions from WPC and updates Trasactions Object with
     * new status of each transactions in ITransactions
     * @param transmap - provides the collection of TransactionBean.
     * @throws WPCMediatorException in case of updation errors/ async. process.
     */
    public abstract Map getTransactionsStatus(String[] strArry, WPSContext context)
        throws WPCMediatorException;

    /**
     * This method fetches Transaction Details primarily the attachment associated with Transaction.
     * List will contain collection of attachment.Attachment is returned as a inputstream.
     * @param transactionId
     * @param wpcUser
     * @return
     * @throws WPCMediatorException
     */
    public List getTransactionsDetails(String transactionId, WPCUser wpcUser)
        throws WPCMediatorException;

    /**
     * This utility method has to be retained in the WPC Mediator.
     * Currently there is only one catalog which needs to be pre-configured for deployment at a client-site.
     * This method and a whole slew of functionality can be created in the Mediator to return the current
     * catalog that a user needs to work with, in the case that there are multiple catalogs to be deployed
     * at a client-site
     * @return String representing the Catalog ID. For example, "CTG_PORTAL_MASTER"
     */
    public abstract String getDefaultCatalog();

    /**
     * This method fetches Image stream from wpc for specific Item (GTNID) and attribute.
     * @param gtnId
     * @param attributePath
     * @param wpcUser
     * @return
     * @throws WPCMediatorException
     */
    public Object getImageObject(FileDownloadResource resourceObj,
        WPSContext context) throws WPCMediatorException;

    /**
     * This Method is for downloading Item(s) from SingleItem or from Search results
     * @param hierarchy
     * @param catalogItem
     * @param headerInfor
     * @param itemPrimaryKey
     * @return
     */
    public abstract String downloadItemDetails(CategoryHierarchy hierarchy,
        ICatalogItem catalogItem, String[] headerInfor,
        String[] itemPrimaryKey, WPSContext context)
        throws WPCMediatorException;

    /**
     * @param context
     * @return
     * @throws WPCMediatorException
     */
    public List getUserRoles(WPSContext context) throws WPCMediatorException;

    /**
     * @param context
     * @return
     * @throws WPCMediatorException
     */
    public List getBusinessProcess(WPSContext context)
        throws WPCMediatorException;

    /**
     * @param context
     * @param roleName
     * @return
     * @throws WPCMediatorException
     */
    public List getBusinessProcess(WPSContext context, String roleName)
        throws WPCMediatorException;

    /**
     * @param context
     * @param roleName
     * @param workflowName
     * @return
     * @throws WPCMediatorException
     */
    public Workflow getSteps(WPSContext context, String roleName,
        String workflowName) throws WPCMediatorException;

    /**
     * @param context
     * @param workflowTo
     * @throws WPCMediatorException
     */
    public void processAction(WPSContext context, WorkFlowTO workflowTo)
        throws WPCMediatorException;
}
