
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.mediator;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPException;

import org.w3c.dom.Element;

import com.ibm.ccd.ui.cache.CacheManager;
import com.ibm.ccd.ui.cache.CacheNotEnabledException;
import com.ibm.ccd.ui.context.IWPCUser;
import com.ibm.ccd.ui.context.WPCUser;
import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.Attachment;
import com.ibm.ccd.ui.dataobjects.CategoryHierarchy;
import com.ibm.ccd.ui.dataobjects.FileDownloadResource;
import com.ibm.ccd.ui.dataobjects.Node;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.items.CatalogItem;
import com.ibm.ccd.ui.dataobjects.items.CatalogItemGenerator;
import com.ibm.ccd.ui.dataobjects.items.CatalogItemParser;
import com.ibm.ccd.ui.dataobjects.items.CatalogNodeXMLHelper;
import com.ibm.ccd.ui.dataobjects.items.ICatalogItem;
import com.ibm.ccd.ui.dataobjects.view.CatalogViewGenerator;
import com.ibm.ccd.ui.dataobjects.view.CatalogViewMaster;
import com.ibm.ccd.ui.dataobjects.view.CatalogViewParser;
import com.ibm.ccd.ui.dataobjects.workflow.PageNavigation;
import com.ibm.ccd.ui.dataobjects.workflow.WorkFlowTO;
import com.ibm.ccd.ui.dataobjects.workflow.Workflow;
import com.ibm.ccd.ui.dataobjects.workflow.WorkflowXMLGenerator;
import com.ibm.ccd.ui.dataobjects.workflow.WorkflowXMLParser;
import com.ibm.ccd.ui.portlets.download.DownloadItemGenerator;
import com.ibm.ccd.ui.portlets.search.core.SearchBean;
import com.ibm.ccd.ui.portlets.workflow.IWorkFlow;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;



/**
 * Implementation for the interface <code>IWPCMediator</code>
 * and provides the functional integration with WPC by exchaning and transforming
 * the Business object from UI to xml for WPC and vice-versa.
 * This is made as singleton class and do not support conversational
 * instance information.
 */
public class WPCMediator implements IWPCMediator {
    public static final String COMP_NAME = "WPCMediator";

    // singleton instance of the mediator
    private static WPCMediator wpcMediator;

    // webservices helper calss
    private WSHelper wsHelper;

    // hashmap to cache the spec
    private HashMap specMap;

    //hashmap to cache the lookup table spec
    private HashMap lookupTableSpecMap;
    private HashMap viewMap;

    /**
    private default constructor
    */
    private WPCMediator() {
    }

    /**
     *  Initializes the Mediator
     */
    public void init() {
        specMap = new HashMap();
        viewMap = new HashMap();
        lookupTableSpecMap = new HashMap();
        wsHelper = new WSHelper();
    }

    /**
     *  Returns the singleton instance of the Mediator.
     */
    public static WPCMediator getInstance() {
        if (wpcMediator == null) {
            synchronized (WSHelper.class) {
                if (wpcMediator == null) {
                    wpcMediator = new WPCMediator();
                    wpcMediator.init();
                }
            }
        }

        return wpcMediator;
    }

    public CategoryHierarchy[] getCatalogs(WPSContext context)
        throws WPCMediatorException {
        WPCUser user = context.getWpcUser();

        // invoke WSHelper to return all catalog 
        Element catalogs = wsHelper.getCatalogs(user);

        return CatalogNodeXMLHelper.parseCategoryNode(catalogs);
    }

    /**
     *  returns the children of the catalog node,
     *  @see IWPCMediator
     */
    public Node[] getCatalogChildren(String parentID, WPSContext context)
        throws WPCMediatorException {
        WPCUser user = null;
        user = context.getWpcUser();

        // prepare the request XML
        String reqXML = CatalogNodeXMLHelper.prepareCtgChilrenRequestXML(parentID);

        // invoke WSHelper to return the chilren element which groups the child nodes.
        Element childrenNode = wsHelper.getCatalogChildren(reqXML, user);

        if (childrenNode == null) {
            return null;
        }

        return CatalogNodeXMLHelper.parseParentNode(childrenNode);
    }

    /**
     *  returns the children of the catalog node,
     *  @see IWPCMediator
     */
    public Node[] getCatalogChildren(String parentCategory, String catalogName,
        String hierarchyName, String heirarchyType, String childrenSize,
        WPSContext context) throws WPCMediatorException {
        WPCUser user = null;
        user = context.getWpcUser();

        // prepare the request XML
        String reqXML = CatalogNodeXMLHelper.prepareCtgChilrenRequestXML(parentCategory,
                catalogName, hierarchyName, heirarchyType, childrenSize);

        // invoke WSHelper to return the chilren element which groups the child nodes.
        Element childrenNode = wsHelper.getCatalogChildren(reqXML, user);

        if (childrenNode == null) {
            return null;
        }

        return CatalogNodeXMLHelper.parseParentNode(childrenNode);
    }

    /**
            * Returns the CatalogItem Spec. for the given catalogID and parentCategory
            * @see IWPCMediator#getCatalogItem(String, String)
            */
    public ICatalogItem getCatalogItem(String parentCgyID, String catalogID,
        WPSContext context) throws WPCMediatorException {
    	
    	long begin = System.currentTimeMillis();
        WPCUser user = context.getWpcUser();

        // check if the spec element is cached in the system.
        String key = (String) user.getProperty(IWPCUser.KEY_USER_ID) + "__";
        
        String groupKey = "ITEM_SKELETON";
        
        if (UIUtils.isEmpty(parentCgyID) || parentCgyID == "/") {
            key = key + catalogID;
        } else {
            key = key + catalogID + "__" + parentCgyID;
        }
        
        
        /* Added for including locale info with the key */
        if(! UIUtils.isEmpty(IWPCUser.KEY_WPC_BROWSER_LOCALE))
        key = key + "__" +  user.getProperty(IWPCUser.KEY_WPC_BROWSER_LOCALE);


        System.out.println("groupKey " + groupKey + " :: " + key);	
//      Element specElement = (Element) specMap.get(key);
      Element specElement = null;
      
      //Added by Suresh
      CacheManager cManager = null;
      try{
      	cManager = CacheManager.getInstance();
      } catch(CacheNotEnabledException e){
      	e.printStackTrace();
      }

      specElement = cManager != null ? (Element) cManager.getFromCache(groupKey,key) : null;
      System.out.println("The specElement is :: " + specElement);

		//	Always fetch spec.Never cache it because of localization.
        if (specElement == null) { 
            // first time invokation try to get from WebService call
            if (IReference.COND_CMPL) {
                System.out.println(COMP_NAME +
                    ":: SpecElement Not found in the cache , invoking the webservice call");
            }

            // construct the xml
            String catalogReqXML = CatalogItemGenerator.prepareCatalogSpecRequestXML(parentCgyID,
                    catalogID);
            
            try {
               // user = context.getWpcUser();

                // call WSHelper..
                specElement = wsHelper.getCatalogSpecXML(catalogReqXML, user);
            } catch (WPCMediatorException wpce) {
                wpce.printStackTrace();
                throw wpce;
            } catch (Exception e) {
                e.printStackTrace();
                throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR,
                    e);
            }
            
//          Put the spec element in cache
            try{
            	System.out.println("After putting in cache :: " + specElement);
            	cManager.putInCache(groupKey, key, specElement);
            } catch(Exception e){
            }
        }
        
        //specMap.put(key, specElement);
        long end = System.currentTimeMillis();
        
        System.out.println("Time Difference " + (end - begin));

        //return the catalog item 
        return CatalogItemParser.getCatalogSpecItem(specElement);
    }

    /**
    * Returns the CatalogItem  for the given  itemID,catalogID and parentCategory
    * @see IWPCMediator#getCatalogItem(String, String, String)
    */
    public ICatalogItem getCatalogItem(String parentCgyID, String catalogID,
        String itemID, WPSContext context) throws WPCMediatorException {
        ICatalogItem item;

        try {
            // get the spec.
            item = getCatalogItem(parentCgyID, catalogID, context);

            //construct the item xml 
            String itemReqXML = CatalogItemGenerator.prepareItemDetailsRequestXML(parentCgyID,
                    catalogID, itemID);
            WPCUser user = context.getWpcUser();

            // call the soapHelper			
            Element itemElement = wsHelper.getCatalogItemXML(itemReqXML, user);

            // prepare the item by parsing the results          
            CatalogItemParser.prepareItem(item, itemElement);

            return item;
        } catch (WPCMediatorException wpce) {
            wpce.printStackTrace();
            throw wpce;
        } catch (Exception e) {
            e.printStackTrace();
            throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR, e);
        }
    }

	/**
	 * returs a list of CatalogViews for a given CatalogName
	 * @param catalogName
	 * @param context
	 * @return
	 * @throws WPCMediatorException
	 */
	public List getCatalogViewList(
		String catalogName,
		WPSContext context)
		throws WPCMediatorException{
			
			String reqXML =
				CatalogViewGenerator.prepareCatalogViewListReqXML(catalogName);
			try {
				WPCUser user = context.getWpcUser();

				Element res = wsHelper.getCatalogViewListXML(reqXML, user);

				return CatalogViewParser.getCatalogViewNameList(res);

			} catch (WPCMediatorException wpc) {
				wpc.printStackTrace();
				throw wpc;
			} catch (Exception e) {
				e.printStackTrace();
				throw new WPCMediatorException(
					WPCMediatorException.SYSTEM_ERROR,
					e);
			}
		}

	/**
	 * 
	 * @param catalogName
	 * @param catalogViewName
	 * @param context
	 * @return
	 * @throws WPCMediatorException
	 */
	public CatalogViewMaster getCatalogView(
		String catalogName,
		String catalogViewName,
		WPSContext context)
		throws WPCMediatorException {

		String reqXML =
			CatalogViewGenerator.prepareCatalogViewReqXML(
				catalogName,
				catalogViewName);
		try {
			WPCUser user = context.getWpcUser();

			Element res = wsHelper.getCatalogViewXML(reqXML, user);

			return CatalogViewParser.getCatalogView(res);

		} catch (WPCMediatorException wpc) {
			wpc.printStackTrace();
			throw wpc;
		} catch (Exception e) {
			e.printStackTrace();
			throw new WPCMediatorException(
				WPCMediatorException.SYSTEM_ERROR,
				e);
		}
	}

	/**
	 * 
	 * @param workFlowName
	 * @param stepName
	 * @param context
	 * @return
	 * @throws WPCMediatorException
	 */
	public CatalogViewMaster getWorkFlowView(
		String workFlowName,
		String stepName,
		WPSContext context)
		throws WPCMediatorException {

		String reqXML =
			CatalogViewGenerator.prepareWorkFlowViewReqXML(
				workFlowName,
				stepName);
		try {
			WPCUser user = context.getWpcUser();

			Element res = wsHelper.getCatalogViewXML(reqXML, user);

			return CatalogViewParser.getWorkFlowView(res);

		} catch (WPCMediatorException wpc) {
			wpc.printStackTrace();
			throw wpc;
		} catch (Exception e) {
			e.printStackTrace();
			throw new WPCMediatorException(
				WPCMediatorException.SYSTEM_ERROR,
				e);
		}
	}
	

    /**
    * Creates the Item in WPC and returns the itemID(primary key)
    * @see IWPCMediator#createItem(ICatalogItem)
    */
    public String createItem(ICatalogItem item, WPSContext context)
        throws WPCMediatorException {
        // construct the XML for the item to create
        String reqXML = CatalogItemGenerator.prepareItemAddRequestXML(item);

        try {
            WPCUser user = context.getWpcUser();

            // invoke the WSHelper
            return wsHelper.createItem(reqXML, user);
        } catch (WPCMediatorException wpce) {
            wpce.printStackTrace();
            throw wpce;
        } catch (Exception e) {
            e.printStackTrace();
            throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR, e);
        }
    }

	/**
	* Creates the Item in WPC and returns the itemID(primary key)
	*/
	public String cloneItem(ICatalogItem item, WPSContext context)
		throws WPCMediatorException {
		// construct the XML for the item to create

		CatalogItem citem = (CatalogItem)item;
		Map map = citem.getFlatAttributeMap();
		
		Enumeration enumValues = Collections.enumeration(map.values());
		while(enumValues.hasMoreElements()) {
			Attribute attrib = (Attribute) enumValues.nextElement();
			attrib.setParentGroupFlagValueChanged();
			attrib.setValueChanged(true);
		}
		
		try {
			return createItem(citem,context);
		} catch (WPCMediatorException wpce) {
			wpce.printStackTrace();
			throw wpce;
		} catch (Exception e) {
			e.printStackTrace();
			throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR, e);
		}
	}

    /**
    * Updates the Item in WPC.
    * @see IWPCMediator#updateItem(ICatalogItem)
    */
    public void updateItem(ICatalogItem item, WPSContext context)
        throws WPCMediatorException {
        // construct the xml for the item to be updated
        String reqXML = CatalogItemGenerator.prepareItemUpdateRequestXML(item);

        try {
            WPCUser user = context.getWpcUser();

            // call the WSHelper
            wsHelper.updateItem(reqXML, user);
        } catch (WPCMediatorException wpce) {
            wpce.printStackTrace();
            throw wpce;
        } catch (Exception e) {
            e.printStackTrace();
            throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR, e);
        }
    }

    /**
     * Deletes the Item in WPC.
     * @see IWPCMediator#deleteItem(ICatalogItem)
     */
    public void deleteItem(ICatalogItem item, WPSContext context)
        throws WPCMediatorException {
        deleteItem(item.getParentCategoryID(), item.getCatalogID(),
            item.getItemID(), context);
    }

    /**
    * Deletes the Item in WPC.
    * @see IWPCMediator#deleteItem(ICatalogItem)
    */
    public void deleteItem(String parentCgy, String catalogID, String itemID,
        WPSContext context) throws WPCMediatorException {
        String itemDeleteReqXML = CatalogItemGenerator.prepareItemDeleteRequestXML(parentCgy,
                catalogID, itemID);

        try {
            WPCUser user = context.getWpcUser();
            wsHelper.deleteItem(itemDeleteReqXML, user);
        } catch (WPCMediatorException wpce) {
            wpce.printStackTrace();
            throw wpce;
        } catch (Exception e) {
            e.printStackTrace();
            throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR, e);
        }
    }

    /**
     * Returns Clone of the CatalogItem  for the given  item
     * @see IWPCMediator#getCatalogCloneItem(String, String, String)
     */
    public ICatalogItem getCatalogItemClone(ICatalogItem item,
        WPSContext context) throws WPCMediatorException {
        return getCatalogItemClone(item.getParentCategoryID(),
            item.getCatalogID(), item.getItemID(), context);
    }

    /**
     * Returns Clone of the CatalogItem  for the given  itemID,catalogID and parentCategory
     * @see IWPCMediator#getCatalogCloneItem(String, String, String)
     */
    public ICatalogItem getCatalogItemClone(String parentCgyID,
        String catalogID, String itemID, WPSContext context)
        throws WPCMediatorException {
        ICatalogItem item;

        try {
            // get the spec.
            item = getCatalogItem(parentCgyID, catalogID, context);

            //construct the item xml 
            String itemCloneReqXML = CatalogItemGenerator.prepareItemCloneRequestXML(parentCgyID,
                    catalogID, itemID);
            WPCUser user = context.getWpcUser();

            // call the soapHelper			
            Element itemElement = wsHelper.getCatalogItemCloneXML(itemCloneReqXML,
                    user);

            // prepare the item by parsing the results          
            CatalogItemParser.prepareItem(item, itemElement);

            return item;
        } catch (WPCMediatorException wpce) {
            wpce.printStackTrace();
            throw wpce;
        } catch (Exception e) {
            e.printStackTrace();
            throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR, e);
        }
    }

    /**
     * Uploads the file into WPC, and returns with the document path where the file has been uploaded.
     * @see IWPCMediator#uploadFile(Attachment)
     */
    public String uploadFile(Attachment attachment, WPSContext context)
        throws WPCMediatorException {
        WPCUser user = context.getWpcUser();
        String documentPath = wsHelper.uploadFile("", attachment, user);

        return documentPath;
    }

    /**
     *  Request WPC to trigger uploaded file for an update.
     *  @see IWPCMediator#processUploadedFile(String)
     */
    public String processUploadedFile(String documentPath, String collabName,
        WPSContext context) throws WPCMediatorException {
        WPCUser user = context.getWpcUser();

        if (UIUtils.isEmpty(collabName)) {
            collabName = UIUtils.getCatalogName(user);
        }

        return wsHelper.processUploadedFile(documentPath, collabName, user);
    }

    /**
     *  Request WPC to perform the search
     *  @see IWPCMediator#performSearch(SearchBean)
     */
    public void performSearch(SearchBean searchBean, WPSContext context)
        throws WPCMediatorException {
        String searchReqXML = searchBean.getReqXML();
        long startIndex=searchBean.getStartIndex();
		long endIndex=searchBean.getEndIndex();
        WPCUser user = context.getWpcUser();
        String responseXML = wsHelper.getSearchCriteriaSpecXML(searchReqXML,
                user, startIndex,endIndex);
        searchBean.setResponseXML(responseXML);
    }


    /**
     * @param searchBean
     * @param context
     * @throws WPCMediatorException
     */
    public void performSearchForMultiEdit(SearchBean searchBean, WPSContext context)
        throws WPCMediatorException {
        String searchReqXML = searchBean.getReqXML();
        String maxResults = searchBean.getMaxResults();
        WPCUser user = context.getWpcUser();
        String responseXML = wsHelper.getSearchCriteriaSpecXMLForMultiEdit(searchReqXML,
                user, maxResults,searchBean.getCollabName());
        searchBean.setResponseXML(responseXML);
    }
    /**
     *  Request WPC to perform the search
     *  @see IWPCMediator#performSearch(SearchBean)
     */
    public void performLookupTableSearch(SearchBean searchBean,
        WPSContext context) throws WPCMediatorException {
        String searchReqXML = searchBean.getReqXML();
        
		long startIndex=searchBean.getStartIndex();
		long endIndex=searchBean.getEndIndex();
        WPCUser user = context.getWpcUser();
        String responseXML = wsHelper.getLookupTableSearchCriteriaSpecXML(searchReqXML,
                user, startIndex,endIndex);
        searchBean.setResponseXML(responseXML);
    }

    /**
     * Fetching the Transactions status from WPC
     * @param transmap
     * @return
     * @throws WPCMediatorException
     */
    public Map getTransactionsStatus(String[] transkeyArray, WPSContext context)
        throws WPCMediatorException {
        String commanSepkeys = UIUtils.getCommaSeparatedId(transkeyArray);
        WPCUser user = context.getWpcUser();
        String responseString = wsHelper.getTransactionsStatus(commanSepkeys,
                user);
        Map map = convertToMap(responseString);

        return map;
    }

    private Map convertToMap(String strXml) {
        HashMap hshMap = new HashMap();
        List list = UIUtils.tokenizeString(strXml, ",");
        Iterator iterator = list.iterator();

        while (iterator.hasNext()) {
            String dataSet = (String) iterator.next();
            List listIdStatus = UIUtils.tokenizeString(dataSet, ":");
            hshMap.put(((String) listIdStatus.get(0)).trim(),
                ((String) listIdStatus.get(1)).trim());
        }

        return hshMap;
    }

    /**
     * Used for fetching Transaction details and attachment
     * @param transactionId
     * @param wpcUser
     * @return
     * @throws WPCMediatorException
     */
    public List getTransactionsDetails(String transactionId, WPCUser wpcUser)
        throws WPCMediatorException {
        List attachmentList = wsHelper.getTransactionsDetails(transactionId,
                wpcUser);

        return attachmentList;
    }

    /**
    * Returns the List of CatalogItem Spec. for the given array of comman separated catalogID
    * @param arrStr String
    */
    public Map getCatalogItems(List idlist, String catalogId, WPSContext context)
        throws WPCMediatorException {
        HashMap catListMap = new HashMap();
        ArrayList retList = new ArrayList();
        Set unSucc = new HashSet();
        Iterator iterator = idlist.iterator();
        WPCMediatorException wpcExcep = null;

        while (iterator.hasNext()) {
            String itemId = (String) iterator.next();
            ICatalogItem catalogItem = null;

            try {
                catalogItem = getCatalogItem("", catalogId, itemId.trim(),
                        context);
                retList.add(catalogItem);
            } catch (WPCMediatorException e) {
                unSucc.add(itemId);
                e.printStackTrace();
                wpcExcep = e;
            }
        }

        catListMap.put(IReference.MASSUPDATE_MAP_LIST_KEY, retList);
        catListMap.put(IReference.MASSUPDATE_MAP_MESSAGE_KEY, unSucc);

        // If 0 items are fetched then mediator will throw Exception to track and display the reason
        if (retList.size() == 0) {
            throw wpcExcep;
        }

        //return the catalog item Map
        return catListMap;
    }

    /**
     * @see IWPCMediator
     */
    public String getDefaultCatalog() {
        return (String) ConfigManager.getInstance().getFrameworkConfig().get(IReference.PRIMARY_CATALOG);
    }

    /**
     * @param context WPSContext
     * @param resourceObj FileDownloadResource
     */
    public Object getImageObject(FileDownloadResource resourceObj,
        WPSContext context) throws WPCMediatorException {
        Object imageObject = wsHelper.getImageObject(resourceObj,
                context.getWpcUser());
        AttachmentPart attachPart = (AttachmentPart) imageObject;
        Object contentObject = null;

        try {
            contentObject = attachPart.getContent();
        } catch (SOAPException e) {
            e.printStackTrace();
        }

        return contentObject;
    }

    /**
     *
     * @see com.ibm.ccd.ui.mediator.IWPCMediator#downloadItemDetails(com.ibm.ccd.ui.dataobjects.CategoryHierarchy, com.ibm.ccd.ui.dataobjects.items.ICatalogItem, java.lang.String[], java.lang.String[])
     */
    public String downloadItemDetails(CategoryHierarchy hierarchy,
        ICatalogItem catalogItem, String[] headerInfor,
        String[] itemPrimaryKey, WPSContext context)
        throws WPCMediatorException {
        WPCUser wpcUser = context.getWpcUser();

        String requestXML = DownloadItemGenerator.prepareDownloadItemRequestXML(hierarchy,
                catalogItem, headerInfor, itemPrimaryKey);

        return wsHelper.downloadItemDetails(requestXML, wpcUser);
    }

    /**
     * @param context
     * @return
     * @throws WPCMediatorException
     */
    public List getUserRoles(WPSContext context) throws WPCMediatorException {
        Element[] element = wsHelper.getUserRole(context.getWpcUser());
        List roleList = WorkflowXMLParser.getUserRolesList(element);

        return roleList;
    }

    /**
     * @param context
     * @param roleName
     * @return
     * @throws WPCMediatorException
     */
    public List getBusinessProcess(WPSContext context, String roleName)
        throws WPCMediatorException {
        String reqXML = WorkflowXMLGenerator.prepareXMLWorkflowsRole(roleName);
        Element element = wsHelper.getBusinessProcess(context.getWpcUser(),
                reqXML);

        return WorkflowXMLParser.getBusinessProcessNamesList(element);
    }

    /**
     * @param user
     * @return
     * @throws WPCMediatorException
     */
    public List getBusinessProcess(WPSContext context)
        throws WPCMediatorException {
        Element element = wsHelper.getBusinessProcess(context.getWpcUser(), "");

        return WorkflowXMLParser.getBusinessProcessNamesList(element);
    }

    /**
     * @param context
     * @param workflowName
     * @return
     * @throws WPCMediatorException
     */
    public Workflow getSteps(WPSContext context, String roleName,
        String workflowName) throws WPCMediatorException {
        String reqXML = WorkflowXMLGenerator.prepareXMLWorkflowsColDetails(workflowName,
                roleName);
        Element element = wsHelper.getSteps(context.getWpcUser(), reqXML);

        return WorkflowXMLParser.getStepsList(element);
    }

    /**
     * @param context
     * @param roleName
     * @param collabName
     * @param workflowName
     * @param stepName
     * @param categoryFilter 
     * @param pageNav
     * @return
     * @throws WPCMediatorException
     */
    public Map getStepItems(WPSContext context, String roleName,
        String collabName, String workflowName, String stepName,String categoryFilter,
        PageNavigation pageNav, List attributeList) throws WPCMediatorException {
        String reqXML = WorkflowXMLGenerator.prepareXMLStepItems(roleName,
                collabName, workflowName, stepName, categoryFilter, pageNav, attributeList);
        Element collabAreaElement = wsHelper.getStepItems(context.getWpcUser(),
                reqXML);

        // Getting Items list
        Map map = WorkflowXMLParser.getItemsMap(collabAreaElement);

        return map;
    }
    /**
     * @param context
     * @param roleName
     * @param collabName
     * @param workflowName
     * @param stepName
     * @param pageNav
     * @return
     * @throws WPCMediatorException
     */
    public Map getStepItems(WPSContext context, String roleName,
        String collabName, String workflowName, String stepName,
        PageNavigation pageNav, List attributeList) throws WPCMediatorException {
        
        return getStepItems(context,roleName,collabName,workflowName,stepName,
                "",pageNav,attributeList);
    }

    /**
     * For initiating  the process Actions
     * @param context
     * @param roleName
     * @param items
     * @return
     * @throws WPCMediatorException
     */
    public void processAction(WPSContext context, WorkFlowTO workflowTo)
        throws WPCMediatorException {
        String action = "";

        if ((workflowTo.getCommand().equals(IWorkFlow.RESERVE_ITEM)) ||
                (workflowTo.getCommand().equals(IWorkFlow.RESERVE_EDIT_ITEM))) {
            action = ISoapMessageKeys.WORKFLOW_ACTION_RESERVE;
        } else if (workflowTo.getCommand().equals(IWorkFlow.UNRESERVE_ITEM)) {
            action = ISoapMessageKeys.WORKFLOW_ACTION_UNRESERVE;
        } else if (workflowTo.getCommand().equals(IWorkFlow.NEXT_STEP_ITEM)) {
            action = ISoapMessageKeys.WORKFLOW_ACTION_MOVE;
        }

        String reqXML = WorkflowXMLGenerator.prepareXMLItemAction(workflowTo.getArrSelectedItem(),
                workflowTo.getCollabAreaListContainer().getSelectedValue(),
                workflowTo.getStepWorkflow().getWorkflowName(),
                workflowTo.getStepWorkflow().getSelectedStep().getStepName(),
                workflowTo.getNextStepListContainer().getSelectedValue(), action);

        wsHelper.processAction(context.getWpcUser(), reqXML);
    }
    
	public Attribute[] getSearchAttributes(String catalogId,WPSContext context) throws WPCMediatorException {
		WPCUser user = null;
		Element specElement = null;
			// construct the xml
		String searchSpecReqXML = CatalogItemGenerator.prepareCatalogSpecRequestXML(null,catalogId);

			try {
				user = context.getWpcUser();
				// call WSHelper..
				specElement = wsHelper.getSearchSpecXML(searchSpecReqXML,user);
			} catch (WPCMediatorException wpce) {
				wpce.printStackTrace();
				throw wpce;
			} catch (Exception e) {
				e.printStackTrace();
				throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR,
					e);
			}
		//return the catalog item 
		return CatalogItemParser.getSpecAttributes(specElement);

	}


    /**
     * @deprecated same operationis done by getSearchAttributes() method.
      * Returns the CatalogItem Spec. for the given catalogID and parentCategory
      * @see IWPCMediator#getCatalogItem(String, String)
      */
    public Attribute[] getLookupTableAttributes(String lookupID,
        	WPSContext context) throws WPCMediatorException {
        	WPCUser user = null;
        	Element specElement =null; 
           // construct the xml
            String lookupTableSpecReqXML = CatalogItemGenerator.prepareLookupTableSpecRequestXML(lookupID);

            try {
                user = context.getWpcUser();
                // call WSHelper..
                specElement = wsHelper.getLookupTableSpecXML(lookupTableSpecReqXML,user);
            } catch (WPCMediatorException wpce) {
                wpce.printStackTrace();
                throw wpce;
            } catch (Exception e) {
                e.printStackTrace();
                throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR,
                    e);
            }

        //return the catalog item 
        return CatalogItemParser.getSpecAttributes(specElement);
    }

	public Attribute[] getSearchResultAttributes(String catalogId,WPSContext context) throws WPCMediatorException {
		WPCUser user = null;
		Element specElement = null;
			// construct the xml
		String searchSpecReqXML = CatalogItemGenerator.prepareCatalogSpecRequestXML(null,catalogId);

			try {
				user = context.getWpcUser();
				// call WSHelper..
				specElement = wsHelper.getSearchResultSpecXML(searchSpecReqXML,user);
			} catch (WPCMediatorException wpce) {
				wpce.printStackTrace();
				throw wpce;
			} catch (Exception e) {
				e.printStackTrace();
				throw new WPCMediatorException(WPCMediatorException.SYSTEM_ERROR,
					e);
			}
		//return the catalog item 
		return CatalogItemParser.getSpecAttributes(specElement);

	}
}
