
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Nov 26, 2004
 */
package com.ibm.ccd.ui.portlets.fileupload;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.servlet.ServletRequest;


/**
 * This class parsers the  MultipartRequest and returns the file
 */
public class MultipartRequest {
    /**
     * Indicates the max post size
     */
    private static final int DEFAULT_MAX_POST_SIZE = 1024 * 1024 * 1024;

    // 1 Meg

    /**
     * contains the <code>HttpServletRequest</code>
     */
    private ServletRequest req;

    /**
     * indicates the max allowed size
     */
    private int maxSize;

    /**
     * contains request parameters
     */
    private Hashtable parameters = new Hashtable(); // name - value

    /**
     * contains the files
     */
    private Hashtable files = new Hashtable(); // name - UploadedFile

    /**
     * Constructor
     * @param request
     * @throws IOException
     */
    public MultipartRequest(ServletRequest request) throws IOException {
        this(request, DEFAULT_MAX_POST_SIZE);
    }

    /**
     * constructor
     * @param request
     * @param maxPostSize
     * @throws IOException
     */
    public MultipartRequest(ServletRequest request, int maxPostSize)
        throws IOException {
        // Sanity check values
        if (request == null) {
            throw new IllegalArgumentException("request cannot be null");
        }

        if (maxPostSize <= 0) {
            throw new IllegalArgumentException("maxPostSize must be positive");
        }

        // Save the request, dir, and max size
        req = request;

        maxSize = maxPostSize;

        readRequest();
    }

    /**
     *
     * @return Enumeration
     */
    public Enumeration getParameterNames() {
        return parameters.keys();
    }

    /**
     * @return Enumeration
     */
    public Enumeration getFileNames() {
        return files.keys();
    }

    /**
     *
     * @param name
     * @return
     */
    public String getParameter(String name) {
        try {
            String param = (String) parameters.get(name);

            if (param.equals("")) {
                return null;
            }

            return param;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     *
     * @param name
     * @return
     */
    public String getFilesystemName(String name) {
        try {
            UploadedFile file = (UploadedFile) files.get(name);

            return file.getFilesystemName(); // may be null
        } catch (Exception e) {
            return null;
        }
    }

    /**
     *
     * @param name
     * @return
     */
    public String getContentType(String name) {
        try {
            UploadedFile file = (UploadedFile) files.get(name);

            return file.getContentType(); // may be null
        } catch (Exception e) {
            return null;
        }
    }

    /**
     *
     * @param name
     * @return
     */
    public File getFile(String name) {
        try {
            UploadedFile file = (UploadedFile) files.get(name);

            return file.getFile(); // may be null
        } catch (Exception e) {
            return null;
        }
    }

    /**
     *
     * @throws IOException
     */
    protected void readRequest() throws IOException {
        // Check the content type to make sure it's "multipart/form-data"
        String type = req.getContentType();

        if ((type == null) ||
                !type.toLowerCase().startsWith("multipart/form-data")) {
            throw new IOException(
                "Posted content type isn't multipart/form-data");
        }

        // Check the content length to prevent denial of service attacks
        int length = req.getContentLength();

        if (length > maxSize) {
            throw new IOException("Posted content length of " + length +
                " exceeds limit of " + maxSize);
        }

        // Get the boundary string; it's included in the content type.
        // Should look something like "------------------------12012133613061"
        String boundary = extractBoundary(type);

        if (boundary == null) {
            throw new IOException("Separation boundary was not specified");
        }

        // Construct the special input stream we'll read from
        MultipartInputStreamHandler in = new MultipartInputStreamHandler(req.getInputStream(),
                boundary, length);

        // Read the first line, should be the first boundary
        String line = in.readLine();

        if (line == null) {
            throw new IOException("Corrupt form data: premature ending");
        }

        // Verify that the line is the boundary
        if (!line.startsWith(boundary)) {
            throw new IOException("Corrupt form data: no leading boundary");
        }

        // Now that we're just beyond the first boundary, loop over each part
        boolean done = false;

        while (!done) {
            done = readNextPart(in, boundary);
        }
    }

    /**
     *
     * @param in
     * @param boundary
     * @return
     * @throws IOException
     */
    protected boolean readNextPart(MultipartInputStreamHandler in,
        String boundary) throws IOException {
        // Read the first line, should look like this:
        // content-disposition: form-data; name="field1"; filename="file1.txt"
        String line = in.readLine();

        if (line == null) {
            // No parts left, we're done
            return true;
        }

        // Parse the content-disposition line
        String[] dispInfo = extractDispositionInfo(line);
        String disposition = dispInfo[0];
        String name = dispInfo[1];
        String filename = dispInfo[2];

        // Now onto the next line. This will either be empty 
        // or contain a Content-Type and then an empty line.
        line = in.readLine();

        if (line == null) {
            // No parts left, we're done
            return true;
        }

        // Get the content type, or null if none specified
        String contentType = extractContentType(line);

        if (contentType != null) {
            // Eat the empty line
            line = in.readLine();

            if ((line == null) || (line.length() > 0)) { // line should be empty
                throw new IOException("Malformed line after content type: " +
                    line);
            }
        } else {
            // Assume a default content type
            contentType = "application/octet-stream";
        }

        // Now, finally, we read the content (end after reading the boundary)
        if (filename == null) {
            // This is a parameter
            String value = readParameter(in, boundary);
            parameters.put(name, value);
        } else {
            // This is a file
            readAndSaveFile(in, boundary, filename);
            files.put(filename, new UploadedFile(filename, contentType));
        }

        return false; // there's more to read
    }

    /**
     *
     * @param in
     * @param boundary
     * @return
     * @throws IOException
     */
    protected String readParameter(MultipartInputStreamHandler in,
        String boundary) throws IOException {
        StringBuffer sbuf = new StringBuffer();
        String line;

        while ((line = in.readLine()) != null) {
            if (line.startsWith(boundary)) {
                break;
            }

            sbuf.append(line + "\r\n");

            // add the \r\n in case there are many lines
        }

        if (sbuf.length() == 0) {
            return null; // nothing read
        }

        sbuf.setLength(sbuf.length() - 2); // cut off the last line's \r\n

        return sbuf.toString(); // no URL decoding needed
    }

    protected void readAndSaveFile(MultipartInputStreamHandler in,
        String boundary, String filename) throws IOException {
        long count = 0;
        File f = new File(File.separator + filename);
        FileOutputStream fos = new FileOutputStream(f);
        BufferedOutputStream out = new BufferedOutputStream(fos, 8 * 1024);

        // 8K
        byte[] bbuf = new byte[8 * 1024]; // 8K
        int result;
        String line;

        // ServletInputStream.readLine() has the annoying habit of 
        // adding a \r\n to the end of the last line. 
        // Since we want a byte-for-byte transfer, we have to cut those chars.
        boolean rnflag = false;

        while ((result = in.readLine(bbuf, 0, bbuf.length)) != -1) {
            count++;

            // Check for boundary
            if ((result > 2) && (bbuf[0] == '-') && (bbuf[1] == '-')) { // quick pre-check
                line = new String(bbuf, 0, result, "UTF-8");

                if (line.startsWith(boundary)) {
                    if (count == 2) {
                        throw new IOException();
                    }

                    break;
                }
            }

            // Are we supposed to write \r\n for the last iteration?
            if (rnflag) {
                out.write('\r');
                out.write('\n');
                rnflag = false;
            }

            // Write the buffer, postpone any ending \r\n
            if ((result >= 2) && (bbuf[result - 2] == '\r') &&
                    (bbuf[result - 1] == '\n')) {
                //line = new String(bbuf, 0, result, "ISO-8859-1");
                out.write(bbuf, 0, result - 2); // skip the last 2 chars
                rnflag = true;

                // make a note to write them on the next iteration
            } else {
                out.write(bbuf, 0, result);
            }
        }

        out.flush();
        out.close();
        fos.close();
    }

    /**
     *
     * @param line
     * @return
     */
    private String extractBoundary(String line) {
        int index = line.indexOf("boundary=");

        if (index == -1) {
            return null;
        }

        String boundary = line.substring(index + 9); // 9 for "boundary="

        // The real boundary is always preceded by an extra "--"
        boundary = "--" + boundary;

        return boundary;
    }

    private String[] extractDispositionInfo(String line)
        throws IOException {
        // Return the line's data as an array: disposition, name, filename
        String[] retval = new String[3];

        // Convert the line to a lowercase string without the ending \r\n
        // Keep the original line for error messages and for variable names.
        String origline = line;
        line = origline.toLowerCase();

        // Get the content disposition, should be "form-data"
        int start = line.indexOf("content-disposition: ");
        int end = line.indexOf(";");

        if ((start == -1) || (end == -1)) {
            throw new IOException("Content disposition corrupt: " + origline);
        }

        String disposition = line.substring(start + 21, end);

        if (!disposition.equals("form-data")) {
            throw new IOException("Invalid content disposition: " +
                disposition);
        }

        // Get the field name
        start = line.indexOf("name=\"", end); // start at last semicolon
        end = line.indexOf("\"", start + 7); // skip name=\"

        if ((start == -1) || (end == -1)) {
            throw new IOException("Content disposition corrupt: " + origline);
        }

        String name = origline.substring(start + 6, end);

        // Get the filename, if given
        String filename = null;
        start = line.indexOf("filename=\"", end + 2); // start after name
        end = line.indexOf("\"", start + 10); // skip filename=\"

        if ((start != -1) && (end != -1)) { // note the !=
            filename = origline.substring(start + 10, end);

            // The filename may contain a full path. Cut to just the filename.
            int slash = Math.max(filename.lastIndexOf('/'),
                    filename.lastIndexOf('\\'));

            if (slash > -1) {
                filename = filename.substring(slash + 1); // past last slash
            }

            if (filename.trim().equals("")) {
                throw new IOException();
            }
        }

        // Return a String array: disposition, name, filename
        retval[0] = disposition;
        retval[1] = name;
        retval[2] = filename;

        return retval;
    }

    /**
     *
     * @param line
     * @return
     * @throws IOException
     */
    private String extractContentType(String line) throws IOException {
        String contentType = null;

        // Convert the line to a lowercase string
        String origline = line;
        line = origline.toLowerCase();

        // Get the content type, if any
        if (line.startsWith("content-type")) {
            int start = line.indexOf(" ");

            if (start == -1) {
                throw new IOException("Content type corrupt: " + origline);
            }

            contentType = line.substring(start + 1);
        } else if (line.length() != 0) { // no content type, so should be empty
            throw new IOException("Malformed line after disposition: " +
                origline);
        }

        return contentType;
    }
}
