
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/*
 * Created on Nov 24, 2004
 */
package com.ibm.ccd.ui.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;

import sun.net.TelnetOutputStream;
import sun.net.ftp.FtpClient;

import com.ibm.ccd.ui.context.WPSContext;
import com.ibm.ccd.ui.dataobjects.Attachment;
import com.ibm.ccd.ui.exception.UIException;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.mediator.WPCMediatorException;
import com.ibm.ccd.ui.portlets.fileupload.FileUploadBean;
import com.ibm.ccd.ui.portlets.fileupload.IFileUpload;
import com.ibm.ccd.ui.portlets.fileupload.MultipartRequest;


/**
  *This is the file upload utility class used for uploading files to the
 * WPC from WPS.<p>
 * This class can provide various implementations for uploading the file. namely FTP, SOAP,MQ and SMTP.
 */
public class FileUploadUtil {
    /**
     * contains the dtat as <code>InputStream</code>
     */
    private InputStream inputStream = null;

    /**
     * Contains file upload confguations
     */
    private FileUploadBean fileUploadBean = null;

    /**
     * Constructor
     * @param inputStream contains data
     * @param pFileUploadBean contains configuration settings
     */
    public FileUploadUtil(InputStream pInputStream,
        FileUploadBean pFileUploadBean) {
        super();
        this.inputStream = pInputStream;
        this.fileUploadBean = pFileUploadBean;
    }

    public static String attachFile(MultipartRequest multi, FileUploadBean bean)
        throws UIException, IOException, FileNotFoundException {
        Enumeration fileList = multi.getFileNames();
        String name = null;
        String fileName = null;
        String contentType = null;
        File file = null;
        String docPath = null;
        FileInputStream inputStream = null;
        WPSContext context = null;

        if (fileList.hasMoreElements()) {
            name = (String) fileList.nextElement();
            fileName = multi.getFilesystemName(name);
            contentType = multi.getContentType(fileName);
            file = multi.getFile(name);
            inputStream = new FileInputStream(file);
        } else {
            throw new UIException("File not Attached ");
        }

        FileUploadBean fileUploadBean = new FileUploadBean();
        fileUploadBean.setProtocol(IFileUpload.PROTOCOL_SOAP);
        fileUploadBean.setFileName(fileName);
        fileUploadBean.setContentType(contentType);

        Attachment attachment = new Attachment();
        attachment.setUploadedFileName(fileName);
        attachment.setMimeType(contentType);
        attachment.setObjectForUpload(inputStream);
        context = bean.getContext();

        WPCMediator mediator = WPCMediator.getInstance();

        try {
            docPath = mediator.uploadFile(attachment, context);
        } catch (WPCMediatorException e) {
            e.printStackTrace();
            throw new UIException(e);
        }

        return docPath;
    }

    /**
     * This method uploads the file to the server depending on the users selection for
     * protocol e.g. FTP, SOAP, MQ etc..
     * @return String
     * @throws WPCMediatorException
     */
    public String uploadFile() throws WPCMediatorException {
        String response = null;
        String protocol = fileUploadBean.getProtocol();

        try {
            validate();

            if (protocol.equals(IFileUpload.PROTOCOL_FTP)) {
                sendFileByFTP(inputStream, fileUploadBean);
            }
        } catch (UIException uiex) {
            uiex.printStackTrace();
        }

        if (protocol.equals(IFileUpload.PROTOCOL_SOAP)) {
            response = sendFileBySOAP(inputStream, fileUploadBean);
        }

        return response;
    }

    /**
     *  This method validates the input before uploading the file
     * @throws UIException
     */
    private void validate() throws UIException {
        int indicator;

        try {
            indicator = inputStream.available();
        } catch (IOException e) {
            throw new UIException(e);
        }

        String protocol = fileUploadBean.getProtocol();

        if ((inputStream == null) || (indicator == 0)) {
            throw new UIException("ERROR:File not found");
        }

        if (ValidationUtil.isNullOrEmpty(protocol)) {
            throw new UIException(
                "ERROR:protocol is missing...can not upload file");
        }
    }

    /**
     * Uploads the file to FTP server
     * @param pInputStream
     * @param pFileUploadBean
     * @throws Exception
     */
    private void sendFileByFTP(InputStream pInputStream,
        FileUploadBean pFileUploadBean) throws UIException {
        // Conatct server and log in
        int byteCount = 0;

        String host = null;
        String user = null;
        String password = null;
        String port = null;
        String folder = null;
        String fileName = null;

        host = pFileUploadBean.getFtpHost();
        user = pFileUploadBean.getFtpUser();
        password = pFileUploadBean.getFtpPassword();
        port = pFileUploadBean.getFtpPort();
        folder = pFileUploadBean.getFtpFolder();
        fileName = pFileUploadBean.getFileName();

        if (ValidationUtil.isNullOrEmpty(host)) {
            throw new UIException("Error:FTP host address not found... ");
        }

        if (ValidationUtil.isNullOrEmpty(user)) {
            user = IFileUpload.DEFAULT_FTP_USER;
            password = IFileUpload.DEFAULT_FTP_PASSWPORD;
        }

        try {
            //connect to ftp server
            FtpClient client = new FtpClient();
            client.openServer(host.trim() + IFileUpload.SEPERATOR_SPACE + port);
            client.login(user, password);

            //set tupe to binary
            client.binary();

            //change dir
            if (!ValidationUtil.isNullOrEmpty(folder)) {
                client.cd(folder);
            }

            // Send the file
            TelnetOutputStream out = client.put(fileName);

            /* Copy one byte at a time from the input stream to the out put stream,
               ending when the read() method returns -1 (which is the signal that
               the end of the stream has been reached.  If any error occurs, print
               an error message.  Also print a message if the file has bee copied
               successfully.  */
            while (true) {
                int data = pInputStream.read();

                if (data < 0) {
                    break;
                }

                out.write(data);
                byteCount++;
            }

            pInputStream.close();
            out.close();

            out.flush();
            out.close();
            pInputStream.close();

            client.closeServer();
        } catch (IOException e) {
            throw new UIException(e);
        } catch (Exception e) {
            throw new UIException(e);
        }
    }

    /**
     * Seds the <code>InputStream</code>
     * @param pInputStream
     * @param pFileUploadBean
     * @return
     * @throws WPCMediatorException
     */
    private String sendFileBySOAP(InputStream pInputStream,
        FileUploadBean pFileUploadBean) throws WPCMediatorException {
        String uploadResponse = null;
        String fileName = null;
        String contentType = null;
        Attachment attachment = null;
        String documentPath = null;
        WPCMediator mediator = null;
        WPSContext context = null;
        fileName = pFileUploadBean.getFileName();
        contentType = pFileUploadBean.getContentType();
        attachment = new Attachment();
        attachment.setMimeType(contentType);
        attachment.setObjectForUpload(pInputStream);
        attachment.setUploadedFileName(fileName);
        mediator = WPCMediator.getInstance();
        context = pFileUploadBean.getContext();
        documentPath = mediator.uploadFile(attachment, context);

        //set the path in bean 
        pFileUploadBean.setDocumentPath(documentPath);
        uploadResponse = mediator.processUploadedFile(documentPath,
                pFileUploadBean.getCollabName(), context);

        return uploadResponse;
    }
}
